@@ -23,7 +23,8 @@
 #include <glib/gi18n.h>
 
 /* FIXME: we should really have a definition in config.h so we can
- * include certain protocol head files */
+ * include certain protocol head files.
+ */
 #include <gossip-jabber.h>
 
 #include "libgossip-marshal.h"
@@ -40,9 +41,9 @@ enum {
 	CONTACT_UPDATED,
 	CONTACT_PRESENCE_UPDATED,
 	CONTACT_REMOVED,
-	COMPOSING_EVENT,
+	COMPOSING,
 
-	/* use to get password from user */
+	/* Used to get password from user. */
 	GET_PASSWORD,
 	
 	SUBSCRIPTION_REQUEST,
@@ -63,6 +64,7 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 	klass->login                = NULL;
 	klass->logout               = NULL;
 	klass->is_connected         = NULL;
+	klass->is_connecting        = NULL;
 	klass->is_valid_username    = NULL;
 	klass->get_example_username = NULL;
 	klass->get_default_server   = NULL;
@@ -92,9 +94,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE, 
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_ACCOUNT);
 
 	/* Maybe include a reason for disconnect? */
 	signals[LOGGED_OUT] = 
@@ -103,9 +105,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT_INT,
 			      G_TYPE_NONE, 
-			      1, G_TYPE_POINTER);
+			      2, GOSSIP_TYPE_ACCOUNT, G_TYPE_INT);
 
 	signals[NEW_MESSAGE] = 
 		g_signal_new ("new-message",
@@ -113,9 +115,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_MESSAGE);
 
 	signals[CONTACT_ADDED] = 
 		g_signal_new ("contact-added",
@@ -123,9 +125,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_CONTACT);
 
 	signals[CONTACT_UPDATED] = 
 		g_signal_new ("contact-updated",
@@ -133,9 +135,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_CONTACT);
 	
 	signals[CONTACT_PRESENCE_UPDATED] =
 		g_signal_new ("contact-presence-updated",
@@ -143,9 +145,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_CONTACT);
 
 	signals[CONTACT_REMOVED] = 
 		g_signal_new ("contact-removed",
@@ -153,19 +155,19 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_CONTACT);
 
-	signals[COMPOSING_EVENT] = 
-		g_signal_new ("composing-event",
+	signals[COMPOSING] = 
+		g_signal_new ("composing",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER_BOOLEAN,
+			      libgossip_marshal_VOID__OBJECT_BOOLEAN,
 			      G_TYPE_NONE,
-			      2, G_TYPE_POINTER, G_TYPE_BOOLEAN);
+			      2, GOSSIP_TYPE_CONTACT, G_TYPE_BOOLEAN);
 
 	signals[GET_PASSWORD] =
 		g_signal_new ("get-password",
@@ -173,9 +175,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_STRING__POINTER,
+			      libgossip_marshal_STRING__OBJECT,
 			      G_TYPE_STRING,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_ACCOUNT);
 
         signals[SUBSCRIPTION_REQUEST] =
                 g_signal_new ("subscription-request",
@@ -183,9 +185,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-                              libgossip_marshal_VOID__POINTER,
+                              libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_CONTACT);
 
 	signals[ERROR] = 
 		g_signal_new ("error",
@@ -193,9 +195,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER_POINTER,
+			      libgossip_marshal_VOID__OBJECT_POINTER,
 			      G_TYPE_NONE,
-			      2, G_TYPE_POINTER, G_TYPE_POINTER);
+			      2, GOSSIP_TYPE_ACCOUNT, G_TYPE_POINTER);
 
 }
 
@@ -278,6 +280,21 @@ gossip_protocol_is_connected (GossipProtocol *protocol)
 	return FALSE;
 }
 
+gboolean
+gossip_protocol_is_connecting (GossipProtocol *protocol)
+{
+	GossipProtocolClass *klass;
+
+	g_return_val_if_fail (GOSSIP_IS_PROTOCOL (protocol), FALSE);
+
+	klass = GOSSIP_PROTOCOL_GET_CLASS (protocol);
+	if (klass->is_connecting) {
+		return klass->is_connecting (protocol);
+	}
+
+	return FALSE;
+}
+
 gboolean
 gossip_protocol_is_valid_username (GossipProtocol *protocol,
 				   const gchar    *username)
