@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004-2005 Imendio AB
+ * Copyright (C) 2004-2006 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,33 +21,31 @@
 #include <config.h>
 #include <stdlib.h>
 #include <string.h>
-#include <glib/gi18n.h>
 
 #include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-debug.h>
+#include <libgossip/gossip-chatroom-contact.h>
 
 #include "gossip-jabber-chatrooms.h"
 #include "gossip-jid.h"
 #include "gossip-jabber-utils.h"
 #include "gossip-jabber-private.h"
 
-#define DEBUG_MSG(x)
-/*#define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
+#define DEBUG_DOMAIN "JabberChatrooms"
 
-#define JOIN_TIMEOUT       20000
-
-#define JOIN_MSG_ID_PREFIX "gc_join_"
+#define XMPP_MUC_USER_XMLNS "http://jabber.org/protocol/muc#user"
 
+#define JOIN_TIMEOUT 20000
 
 struct _GossipJabberChatrooms {
-	GossipJabber   *jabber;
-	GossipPresence *presence;
-	LmConnection   *connection;
+	GossipJabber          *jabber;
+	GossipPresence        *presence;
+	LmConnection          *connection;
 
-	GHashTable     *room_id_hash;
-	GHashTable     *room_jid_hash;
+	GHashTable            *room_id_hash;
+	GHashTable            *room_jid_hash;
 };
 
-
 typedef struct {
 	gint                   ref_count;
 
@@ -68,34 +66,39 @@ typedef struct {
 	LmMessageHandler      *join_handler;
 } JabberChatroom;
 
-
-static JabberChatroom * jabber_chatrooms_chatroom_new         (GossipJabberChatrooms  *chatrooms,
-							       GossipChatroom         *chatroom);
-static JabberChatroom * jabber_chatrooms_chatroom_ref         (JabberChatroom         *room);
-static void             jabber_chatrooms_chatroom_unref       (JabberChatroom         *room);
-static GossipChatroomId jabber_chatrooms_chatroom_get_id      (JabberChatroom         *room);
-static LmHandlerResult  jabber_chatrooms_message_handler      (LmMessageHandler       *handler,
-							       LmConnection           *conn,
-							       LmMessage              *message,
-							       GossipJabberChatrooms  *chatrooms);
-static LmHandlerResult  jabber_chatrooms_presence_handler     (LmMessageHandler       *handler,
-							       LmConnection           *conn,
-							       LmMessage              *message,
-							       GossipJabberChatrooms  *chatrooms);
-static LmHandlerResult  jabber_chatrooms_join_cb              (LmMessageHandler       *handler,
-							       LmConnection           *connection,
-							       LmMessage              *message,
-							       JabberChatroom         *room);
-static GossipContact *  jabber_chatrooms_get_contact          (JabberChatroom         *room,
-							       GossipJID              *jid,
-							       gboolean               *new_contact);
-static void             jabber_chatrooms_get_rooms_foreach    (gpointer                key,
-							       JabberChatroom         *room,
-							       GList                 **list);
-static void             jabber_chatrooms_set_presence_foreach (gpointer                key,
-							       JabberChatroom         *room,
-							       GossipJabberChatrooms  *chatrooms);
-
+static void             jabber_chatrooms_logged_out_cb         (GossipProtocol         *jabber,
+								GossipAccount          *account,
+								gint                    reason,
+								GossipJabberChatrooms  *chatrooms);
+static JabberChatroom * jabber_chatrooms_chatroom_new          (GossipJabberChatrooms  *chatrooms,
+								GossipChatroom         *chatroom);
+static JabberChatroom * jabber_chatrooms_chatroom_ref          (JabberChatroom         *room);
+static void             jabber_chatrooms_chatroom_unref        (JabberChatroom         *room);
+static GossipChatroomId jabber_chatrooms_chatroom_get_id       (JabberChatroom         *room);
+static GossipContact *  jabber_chatrooms_get_contact           (JabberChatroom         *room,
+								GossipJID              *jid,
+								gboolean               *new_contact);
+static LmHandlerResult  jabber_chatrooms_message_handler       (LmMessageHandler       *handler,
+								LmConnection           *conn,
+								LmMessage              *message,
+								GossipJabberChatrooms  *chatrooms);
+static LmHandlerResult  jabber_chatrooms_presence_handler      (LmMessageHandler       *handler,
+								LmConnection           *conn,
+								LmMessage              *message,
+								GossipJabberChatrooms  *chatrooms);
+static LmHandlerResult  jabber_chatrooms_join_cb               (LmMessageHandler       *handler,
+								LmConnection           *connection,
+								LmMessage              *message,
+								JabberChatroom         *room);
+static void             jabber_chatrooms_get_rooms_foreach     (gpointer                key,
+								JabberChatroom         *room,
+								GList                 **list);
+static void             jabber_chatrooms_set_presence_foreach  (gpointer                key,
+								JabberChatroom         *room,
+								GossipJabberChatrooms  *chatrooms);
+static LmMessageNode *  jabber_chatrooms_find_muc_user_node    (LmMessageNode          *parent_node);
+static GossipChatroomRole jabber_chatrooms_get_role            (LmMessageNode          *muc_node);
+static GossipChatroomAffiliation jabber_chatrooms_get_affiliation (LmMessageNode       *muc_node);
 
 GossipJabberChatrooms *
 gossip_jabber_chatrooms_init (GossipJabber *jabber)
@@ -111,15 +114,20 @@ gossip_jabber_chatrooms_init (GossipJabber *jabber)
 
 	chatrooms = g_new0 (GossipJabberChatrooms, 1);
 	
-	chatrooms->jabber     = g_object_ref (jabber);
+	chatrooms->jabber = g_object_ref (jabber);
 	chatrooms->connection = lm_connection_ref (connection);
-	chatrooms->presence   = NULL;
+	chatrooms->presence = NULL;
+
 	chatrooms->room_id_hash = g_hash_table_new (NULL, NULL);
 	chatrooms->room_jid_hash = g_hash_table_new_full (gossip_jid_hash,
 							  gossip_jid_equal,
 							  (GDestroyNotify) gossip_jid_unref,
 							  NULL);
 
+	g_signal_connect (GOSSIP_PROTOCOL (chatrooms->jabber), "logged-out",
+			  G_CALLBACK (jabber_chatrooms_logged_out_cb), 
+			  chatrooms);
+
 	handler = lm_message_handler_new ((LmHandleMessageFunction) jabber_chatrooms_message_handler,
 					  chatrooms, NULL);
 
@@ -146,11 +154,16 @@ gossip_jabber_chatrooms_finalize (GossipJabberChatrooms *chatrooms)
 {
 	if (!chatrooms) {
 		/* We don't error here, because if no connection is
-		made, then we can clean up a GossipJabber object
-		without any chatrooms ever existing */
+		 * made, then we can clean up a GossipJabber object
+		 * without any chatrooms ever existing.
+		 */
 		return;
 	}
 
+	g_signal_handlers_disconnect_by_func (GOSSIP_PROTOCOL (chatrooms->jabber), 
+					      jabber_chatrooms_logged_out_cb, 
+					      chatrooms);
+
 	g_hash_table_destroy (chatrooms->room_id_hash);
 	g_hash_table_destroy (chatrooms->room_jid_hash);
 
@@ -164,6 +177,28 @@ gossip_jabber_chatrooms_finalize (GossipJabberChatrooms *chatrooms)
 	g_free (chatrooms);
 }
 
+static void
+jabber_chatrooms_logged_out_cb (GossipProtocol        *jabber,
+				GossipAccount         *account,
+				gint                   reason,
+				GossipJabberChatrooms *chatrooms)
+{
+	GList *rooms;
+	GList *l;
+
+	/* Clean up chat rooms */
+	rooms = gossip_jabber_chatrooms_get_rooms (chatrooms);
+
+	for (l = rooms; l; l = l->next) {
+		GossipChatroomId id;
+
+		id = GPOINTER_TO_INT (l->data);
+		gossip_jabber_chatrooms_leave (chatrooms, id);
+	}
+
+	g_list_free (rooms);
+}
+
 static JabberChatroom *
 jabber_chatrooms_chatroom_new (GossipJabberChatrooms *chatrooms,
 			       GossipChatroom        *chatroom)
@@ -180,9 +215,8 @@ jabber_chatrooms_chatroom_new (GossipJabberChatrooms *chatrooms,
 	room->ref_count = 1;
 	room->chatroom = g_object_ref (chatroom);
 
-	jid_str = g_strdup_printf ("%s@%s/%s", 
-				   gossip_chatroom_get_room (chatroom), 
-				   gossip_chatroom_get_server (chatroom),
+	jid_str = g_strdup_printf ("%s/%s", 
+				   gossip_chatroom_get_id_str (chatroom), 
 				   gossip_chatroom_get_nick (chatroom));
 	room->jid = gossip_jid_new (jid_str);
 	g_free (jid_str);
@@ -286,18 +320,18 @@ jabber_chatrooms_message_handler (LmMessageHandler      *handler,
 	jid = gossip_jid_new (from);
 
 	room = g_hash_table_lookup (chatrooms->room_jid_hash, jid);
-	id = jabber_chatrooms_chatroom_get_id (room);
-
 	if (!room) {
 		gossip_jid_unref (jid);
 		return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 	}
 	
+	id = jabber_chatrooms_chatroom_get_id (room);
+
 	node = lm_message_node_get_child (m->node, "body");
 	if (node) {
 		if (gossip_jid_get_resource (jid) == NULL) {
 			g_signal_emit_by_name (chatrooms->jabber,
-					       "chatroom-new-room-event",
+					       "chatroom-new-event",
 					       id, node->value);
 		} else {
 			gossip_time_t timestamp;
@@ -311,7 +345,6 @@ jabber_chatrooms_message_handler (LmMessageHandler      *handler,
 			gossip_message_set_timestamp (message, timestamp);
 			
 			contact = jabber_chatrooms_get_contact (room, jid, NULL);
-
 			gossip_message_set_sender (message, contact);
 			gossip_message_set_body (message, node->value);
 			
@@ -338,27 +371,71 @@ jabber_chatrooms_message_handler (LmMessageHandler      *handler,
 
 }
 
+static GossipContact *
+jabber_chatrooms_get_contact (JabberChatroom *room, 
+			      GossipJID      *jid, 
+			      gboolean       *new_contact) 
+{
+	GossipContact *contact;
+	GSList        *l;
+	const gchar   *id;
+	const gchar   *name;
+
+	id = gossip_jid_get_full (jid);
+
+	for (l = room->contacts; l; l = l->next) {
+		contact = GOSSIP_CONTACT (l->data);
+
+		if (g_ascii_strcasecmp (gossip_contact_get_id (contact), id) == 0) {
+			if (new_contact) {
+				*new_contact = FALSE;
+			}
+
+			return contact;
+		}
+	}
+
+	if (new_contact) {
+		*new_contact = TRUE;
+	}
+
+	name = gossip_jid_get_resource (jid);
+	if (!name) {
+		name = id;
+	}
+
+	contact = GOSSIP_CONTACT (gossip_chatroom_contact_new_full (gossip_contact_get_account (room->own_contact),
+								    id, 
+								    name));
+	room->contacts = g_slist_prepend (room->contacts, contact);
+
+	return contact;
+}
+
 static LmHandlerResult
 jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 				   LmConnection          *conn,
 				   LmMessage             *m,
 				   GossipJabberChatrooms *chatrooms)
 {
-	const gchar         *from;
-	GossipJID           *jid;
-	JabberChatroom      *room;
-	GossipContact       *contact;
-	GossipPresence      *presence;
-	GossipPresenceState  p_state;
-	GossipChatroomId     id;
-	LmMessageSubType     type;
-	LmMessageNode       *node;
-	gboolean             new_contact;
-	gboolean             was_offline;
+	const gchar               *from;
+	GossipJID                 *jid;
+	JabberChatroom            *room;
+	GossipContact             *contact;
+	GossipPresence            *presence;
+	GossipPresenceState        p_state;
+	GossipChatroomId           id;
+	LmMessageSubType           type;
+	LmMessageNode             *node;
+	gboolean                   new_contact;
+	gboolean                   was_offline;
+	LmMessageNode             *muc_user_node;
+	GossipChatroomRole         muc_role;
+	GossipChatroomAffiliation  muc_affiliation;
 	
 	from = lm_message_node_get_attribute (m->node, "from");
 	jid = gossip_jid_new (from);
-		
+
 	room = g_hash_table_lookup (chatrooms->room_jid_hash, jid);
 	if (!room) {
 		gossip_jid_unref (jid);
@@ -367,7 +444,7 @@ jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 	
 	id = jabber_chatrooms_chatroom_get_id (room);
 
-	DEBUG_MSG (("ProtocolChatrooms: Presence from: %s", from));
+	gossip_debug (DEBUG_DOMAIN, "Presence from: %s", from);
 
 	type = lm_message_get_sub_type (m);
 	switch (type) {
@@ -379,7 +456,7 @@ jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 		presence = gossip_presence_new ();
  		node = lm_message_node_get_child (m->node, "show");
 		if (node) {
-			p_state = gossip_utils_get_presence_state_from_show_string (node->value);
+			p_state = gossip_jabber_presence_state_from_str (node->value);
 			gossip_presence_set_state (presence, p_state);
 		}
 		node = lm_message_node_get_child (m->node, "status");
@@ -391,16 +468,27 @@ jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 		was_offline = !gossip_contact_is_online (contact);
 		gossip_contact_add_presence (contact, presence);
 
+		muc_user_node = jabber_chatrooms_find_muc_user_node (m->node);
+		muc_role = jabber_chatrooms_get_role (muc_user_node);
+		muc_affiliation = jabber_chatrooms_get_affiliation (muc_user_node);
+
+		g_object_set (contact,
+			      "role", muc_role,
+			      "affiliation", muc_affiliation,
+			      NULL);
+	
+		g_object_unref (presence);
+
 		/* is contact new or updated */
 		if (new_contact || was_offline) {
-			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Presence for new joining contact:'%s'",
-				   id, gossip_jid_get_full (jid)));
+			gossip_debug (DEBUG_DOMAIN, "ID[%d] Presence for new joining contact:'%s'",
+				      id, gossip_jid_get_full (jid));
 			g_signal_emit_by_name (chatrooms->jabber,
 					       "chatroom-contact-joined",
 					       id, contact);
 		} else {
-			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Presence updated for contact:'%s'", 
-				   id, gossip_jid_get_full (jid)));
+			gossip_debug (DEBUG_DOMAIN, "ID[%d] Presence updated for contact:'%s'", 
+				      id, gossip_jid_get_full (jid));
 			g_signal_emit_by_name (chatrooms->jabber,
 					       "chatroom-contact-presence-updated", 
 					       id, contact);
@@ -410,8 +498,8 @@ jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 	case LM_MESSAGE_SUB_TYPE_UNAVAILABLE:
 		contact = jabber_chatrooms_get_contact (room, jid, NULL);
 		if (contact) {
-			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Contact left:'%s'",
-				   id, gossip_jid_get_full (jid)));
+			gossip_debug (DEBUG_DOMAIN, "ID[%d] Contact left:'%s'",
+				      id, gossip_jid_get_full (jid));
 			g_signal_emit_by_name (chatrooms->jabber, 
 					       "chatroom-contact-left",
 					       id, contact);
@@ -420,39 +508,9 @@ jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 		}
 		break;
 
-#if 0
-	case LM_MESSAGE_SUB_TYPE_ERROR: 
-		if (room->callback) {
-			GossipChatroomStatus status;
-
-			status = gossip_chatroom_get_status (room->chatroom);
-
-			if (status == GOSSIP_CHATROOM_CONNECTING) {
-				/* malformed join group chat message is most likely */
-				gossip_chatroom_set_status (room->chatroom, 
-							    GOSSIP_CHATROOM_ERROR);
-
-				/* clean up timeout */
-				if (room->timeout_id) {
-					g_source_remove (room->timeout_id);
-					room->timeout_id = 0;
-				}
-
-				(room->callback) (GOSSIP_CHATROOM_PROVIDER (chatrooms->jabber),
-						  GOSSIP_CHATROOM_JOIN_UNKNOWN_ERROR, 
-						  id, room->user_data);
-
-				/* clean up callback */
-				room->callback = NULL;
-				room->user_data = NULL;
-			}
-		}
-		break;
-#endif
-
 	default:
-		DEBUG_MSG (("ProtocolChatrooms: Presence not handled for:'%s'",
-			   gossip_jid_get_full (jid)));
+		gossip_debug (DEBUG_DOMAIN, "Presence not handled for:'%s'",
+			      gossip_jid_get_full (jid));
 		gossip_jid_unref (jid);
 		return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 	}	
@@ -460,48 +518,6 @@ jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 	return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 }
 
-static GossipContact *
-jabber_chatrooms_get_contact (JabberChatroom *room, 
-			      GossipJID      *jid, 
-			      gboolean       *new_contact) 
-{
-	GSList        *l;
-	GossipContact *c;
-	const gchar   *id;
-
-	id = gossip_jid_get_full (jid);
-
-	if (gossip_jid_equals (jid, room->jid)) {
-		if (new_contact) {
-			*new_contact = FALSE;
-		}
-		return room->own_contact;
-	}
-
-	for (l = room->contacts; l; l = l->next) {
-		c = GOSSIP_CONTACT (l->data);
-
-		if (g_ascii_strcasecmp (gossip_contact_get_id (c), id) == 0) {
-			if (new_contact) {
-				*new_contact = FALSE;
-			}
-			return c;
-		}
-	}
-
-	if (new_contact) {
-		*new_contact = TRUE;
-	}
-
-	c = gossip_contact_new_full (GOSSIP_CONTACT_TYPE_CHATROOM, 
-				     gossip_contact_get_account (room->own_contact),
-				     id, 
-				     gossip_jid_get_resource (jid));
-	room->contacts = g_slist_prepend (room->contacts, c);
-
-	return c;
-}
-
 static gboolean
 jabber_chatrooms_join_timeout_cb (JabberChatroom *room)
 {
@@ -517,7 +533,7 @@ jabber_chatrooms_join_timeout_cb (JabberChatroom *room)
 	}
 
 	id = jabber_chatrooms_chatroom_get_id (room);
-	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Join timed out (internally)", id));
+	gossip_debug (DEBUG_DOMAIN, "ID[%d] Join timed out (internally)", id);
 
 	/* set chatroom status and error */
 	gossip_chatroom_set_status (room->chatroom, GOSSIP_CHATROOM_STATUS_ERROR);
@@ -529,7 +545,7 @@ jabber_chatrooms_join_timeout_cb (JabberChatroom *room)
 	chatrooms = room->chatrooms;
 		
 	if (room->callback != NULL) {
-		DEBUG_MSG (("ProtocolChatrooms: ID[%d] Calling back...", id));
+		gossip_debug (DEBUG_DOMAIN, "ID[%d] Calling back... (timed out)", id);
 		(room->callback) (GOSSIP_CHATROOM_PROVIDER (chatrooms->jabber),
 				  GOSSIP_CHATROOM_JOIN_TIMED_OUT, 
 				  id, room->user_data);
@@ -575,9 +591,9 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
 		/* Duplicate room already exists. */
 		id = jabber_chatrooms_chatroom_get_id (existing_room);
 
-		DEBUG_MSG (("ProtocolChatrooms: ID[%d] Join chatroom:'%s', room already exists.", 
-			   id,
-			   gossip_chatroom_get_room (chatroom)));
+		gossip_debug (DEBUG_DOMAIN, "ID[%d] Join chatroom:'%s', room already exists.", 
+			      id,
+			      gossip_chatroom_get_room (chatroom));
 
 		(callback) (GOSSIP_CHATROOM_PROVIDER (chatrooms->jabber),
 			    GOSSIP_CHATROOM_JOIN_ALREADY_OPEN,
@@ -590,10 +606,10 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
 	/* Get real chatroom. */
 	id = jabber_chatrooms_chatroom_get_id (room);
 
-        DEBUG_MSG (("ProtocolChatrooms: ID[%d] Join chatroom:'%s' on server:'%s'", 
-		   id,
-		   gossip_chatroom_get_room (chatroom),
-		   gossip_chatroom_get_server (chatroom)));
+        gossip_debug (DEBUG_DOMAIN, "ID[%d] Join chatroom:'%s' on server:'%s'", 
+		      id,
+		      gossip_chatroom_get_room (chatroom),
+		      gossip_chatroom_get_server (chatroom));
 
 
 	/* Add timeout for server response. */
@@ -630,7 +646,7 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
                 lm_message_node_add_child (m->node, "show", show);
 	}
 
-        id_str = g_strdup_printf (JOIN_MSG_ID_PREFIX "%d", id);
+        id_str = g_strdup_printf ("muc_join_%d", id);
         lm_message_node_set_attribute (m->node, "id", id_str);
         g_free (id_str);
 
@@ -723,29 +739,32 @@ jabber_chatrooms_join_cb (LmMessageHandler *handler,
 		code = str ? atoi (str) : 0;
 		
 		switch (code) {
-		case 409: {
-			/* conflicting nickname */
-			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Conflicting nickname", id));
-			result = GOSSIP_CHATROOM_JOIN_NICK_IN_USE;
-			break;
-		}
-			
-		case 502: {
-			/* unresolved hostname */
-			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Unable to resolve hostname", id));
+		case 404: 
+ 			/* conflicting nickname */
+			gossip_debug (DEBUG_DOMAIN, "ID[%d] Chatroom not found", id);
 			result = GOSSIP_CHATROOM_JOIN_UNKNOWN_HOST;
 			break;
-		}
 			
-		case 504: {
+		case 409: 
+ 			/* conflicting nickname */
+			gossip_debug (DEBUG_DOMAIN, "ID[%d] Conflicting nickname", id);
+			result = GOSSIP_CHATROOM_JOIN_NICK_IN_USE;
+			break;
+			
+		case 502: 
+			/* unresolved hostname */
+			gossip_debug (DEBUG_DOMAIN, "ID[%d] Unable to resolve hostname", id);
+			result = GOSSIP_CHATROOM_JOIN_UNKNOWN_HOST;
+			break;
+			
+		case 504: 
 			/* remote server timeout */
-			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Join timed out", id));
+			gossip_debug (DEBUG_DOMAIN, "ID[%d] Join timed out", id);
 			result = GOSSIP_CHATROOM_JOIN_TIMED_OUT;
 			break;
-		}
 			
 		default:
-			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Unhandled presence error:%d", id, code));
+			gossip_debug (DEBUG_DOMAIN, "ID[%d] Unhandled presence error:%d", id, code);
 			result = GOSSIP_CHATROOM_JOIN_UNKNOWN_ERROR;
 			break;
 		}
@@ -762,26 +781,25 @@ jabber_chatrooms_join_cb (LmMessageHandler *handler,
 					gossip_chatroom_provider_join_result_as_str (result));
 
 	if (room->callback != NULL) {
-		DEBUG_MSG (("ProtocolChatrooms: ID[%d] Calling back...", id));
+		gossip_debug (DEBUG_DOMAIN, "ID[%d] Calling back...", id);
 		(room->callback) (GOSSIP_CHATROOM_PROVIDER (chatrooms->jabber),
 				  result, id, room->user_data);
 	}
 
-	/* clean up callback data */
+	/* Clean up callback data */
 	room->callback = NULL;
 	room->user_data = NULL;
 
-	/* articulate own contact presence so we appear in group chat */
+	/* Articulate own contact presence so we appear in group chat */
 	if (result == GOSSIP_CHATROOM_JOIN_OK) {
 		gossip_contact_add_presence (room->own_contact, 
 					     chatrooms->presence);
 		
 		g_signal_emit_by_name (chatrooms->jabber,
-				       "chatroom-contact-joined",
-				       id,
-				       room->own_contact);
+				       "chatroom-joined",
+				       id);
 	} else {
-		/* clean up */
+		/* Clean up */
 		g_hash_table_remove (chatrooms->room_id_hash, 
 				     GINT_TO_POINTER (id));
 		g_hash_table_remove (chatrooms->room_jid_hash, 
@@ -802,11 +820,10 @@ gossip_jabber_chatrooms_cancel (GossipJabberChatrooms *chatrooms,
 	room = g_hash_table_lookup (chatrooms->room_id_hash, 
 				    GINT_TO_POINTER (id));
 	if (!room) {
-		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return;
 	}
 
-	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Cancel joining room", id));
+	gossip_debug (DEBUG_DOMAIN, "ID[%d] Cancel joining room", id);
 
 	if (room->timeout_id) {
 		g_source_remove (room->timeout_id);
@@ -823,7 +840,7 @@ gossip_jabber_chatrooms_cancel (GossipJabberChatrooms *chatrooms,
 	gossip_chatroom_set_last_error (room->chatroom, NULL);
 
 	if (room->callback != NULL) {
-		DEBUG_MSG (("ProtocolChatrooms: ID[%d] Calling back...", id));
+		gossip_debug (DEBUG_DOMAIN, "ID[%d] Calling back...", id);
 		(room->callback) (GOSSIP_CHATROOM_PROVIDER (chatrooms->jabber),
 				  GOSSIP_CHATROOM_JOIN_CANCELED, id, room->user_data);
 	}
@@ -849,14 +866,14 @@ gossip_jabber_chatrooms_send (GossipJabberChatrooms *chatrooms,
 	g_return_if_fail (chatrooms != NULL);
 	g_return_if_fail (message != NULL);
 
-	room = (JabberChatroom*) g_hash_table_lookup (chatrooms->room_id_hash, 
-						       GINT_TO_POINTER (id));
+	room = g_hash_table_lookup (chatrooms->room_id_hash, 
+				    GINT_TO_POINTER (id));
 	if (!room) {
 		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return;
 	}
 	
-	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Send message", id));
+	gossip_debug (DEBUG_DOMAIN, "ID[%d] Send message", id);
 	
 	m = lm_message_new_with_sub_type (gossip_jid_get_without_resource (room->jid),
 					  LM_MESSAGE_TYPE_MESSAGE,
@@ -879,15 +896,15 @@ gossip_jabber_chatrooms_change_topic (GossipJabberChatrooms *chatrooms,
 	g_return_if_fail (chatrooms != NULL);
 	g_return_if_fail (new_topic != NULL);
 
-	room = (JabberChatroom*) g_hash_table_lookup (chatrooms->room_id_hash, 
-						       GINT_TO_POINTER (id));
+	room = g_hash_table_lookup (chatrooms->room_id_hash, 
+				    GINT_TO_POINTER (id));
 	if (!room) {
 		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return;
 	}
 
-	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Change topic to:'%s'", 
-		   id, new_topic));
+	gossip_debug (DEBUG_DOMAIN, "ID[%d] Change topic to:'%s'", 
+		      id, new_topic);
 	
 	without_resource = gossip_jid_get_without_resource (room->jid);
 	
@@ -912,15 +929,15 @@ gossip_jabber_chatrooms_change_nick (GossipJabberChatrooms *chatrooms,
 	g_return_if_fail (chatrooms != NULL);
 	g_return_if_fail (new_nick != NULL);
 
-	room = (JabberChatroom*) g_hash_table_lookup (chatrooms->room_id_hash, 
-						       GINT_TO_POINTER (id));
+	room = g_hash_table_lookup (chatrooms->room_id_hash, 
+				    GINT_TO_POINTER (id));
 	if (!room) {
 		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return;
 	}
 
-	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Change chatroom nick to:'%s'", 
-		   id, new_nick));
+	gossip_debug (DEBUG_DOMAIN, "ID[%d] Change chatroom nick to:'%s'", 
+		      id, new_nick);
 
 	gossip_jid_set_resource (room->jid, new_nick);
 	
@@ -940,10 +957,9 @@ gossip_jabber_chatrooms_leave (GossipJabberChatrooms *chatrooms,
 
 	g_return_if_fail (chatrooms != NULL);
 
-	room = (JabberChatroom*) g_hash_table_lookup (chatrooms->room_id_hash, 
-						       GINT_TO_POINTER (id));
+	room = g_hash_table_lookup (chatrooms->room_id_hash, 
+				    GINT_TO_POINTER (id));
 	if (!room) {
-		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return;
 	}
 
@@ -962,65 +978,69 @@ gossip_jabber_chatrooms_leave (GossipJabberChatrooms *chatrooms,
 	g_hash_table_remove (chatrooms->room_jid_hash,
 			     room->jid);
 
-	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Leaving room, ref count is %d", 
-		   id, room->ref_count - 1));
+	gossip_debug (DEBUG_DOMAIN, "ID[%d] Leaving room, ref count is %d", 
+		      id, room->ref_count - 1);
 	
 	jabber_chatrooms_chatroom_unref (room);
 }
 
-const gchar *
-gossip_jabber_chatrooms_get_room_name (GossipJabberChatrooms *chatrooms,
-				       GossipChatroomId       id)
+GossipChatroom *
+gossip_jabber_chatrooms_find (GossipJabberChatrooms *chatrooms,
+			      GossipChatroomId       id)
 {
 	JabberChatroom *room;
 
 	g_return_val_if_fail (chatrooms != NULL, NULL);
 
-	room = (JabberChatroom*) g_hash_table_lookup (chatrooms->room_id_hash,
-						      GINT_TO_POINTER (id));
+	room = g_hash_table_lookup (chatrooms->room_id_hash,
+				    GINT_TO_POINTER (id));
 	if (!room) {
-		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return NULL;
 	}
 
-	return gossip_chatroom_get_name (room->chatroom);
+	return room->chatroom;
 }
 
 void 
 gossip_jabber_chatrooms_invite (GossipJabberChatrooms *chatrooms,
 				GossipChatroomId       id,
-				const gchar           *contact_id,
-				const gchar           *invite)
+				GossipContact         *contact,
+				const gchar           *reason)
 {
 	LmMessage      *m;
-	LmMessageNode  *n;
+	LmMessageNode  *parent;
+	LmMessageNode  *node;
 	JabberChatroom *room;
 
 	g_return_if_fail (chatrooms != NULL);
-	g_return_if_fail (contact_id != NULL);
+	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
+
+	room = g_hash_table_lookup (chatrooms->room_id_hash, 
+				    GINT_TO_POINTER (id));
 
-	room = (JabberChatroom *) g_hash_table_lookup (chatrooms->room_id_hash, 
-						       GINT_TO_POINTER (id));
 	if (!room) {
 		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return;
 	}
 
-	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Inviting contact:'%s'", 
-		   id, contact_id));
+	gossip_debug (DEBUG_DOMAIN, "ID[%d] Invitation to contact:'%s' from:'%s'", 
+		      id, 
+		      gossip_contact_get_id (contact), 
+		      gossip_contact_get_id (room->own_contact));
 
-	m = lm_message_new (contact_id,
+	m = lm_message_new (gossip_jid_get_without_resource (room->jid),
 			    LM_MESSAGE_TYPE_MESSAGE);
-
-	if (invite) {
-		lm_message_node_add_child (m->node, "body", invite); 
-	}
-	
-	n = lm_message_node_add_child (m->node, "x", NULL);
-	lm_message_node_set_attributes (n, 
-					"jid", gossip_jid_get_without_resource (room->jid),
-					"xmlns", "jabber:x:conference",
+	lm_message_node_set_attributes (m->node, 
+					"from", gossip_contact_get_id (room->own_contact), 
 					NULL);
+	
+	parent = lm_message_node_add_child (m->node, "x", NULL);
+	lm_message_node_set_attributes (parent, "xmlns", XMPP_MUC_USER_XMLNS, NULL);
+
+	node = lm_message_node_add_child (parent, "invite", NULL);
+	lm_message_node_set_attributes (node, "to", gossip_contact_get_id (contact), NULL);
+
+	lm_message_node_add_child (node, "reason", reason);
 
 	lm_connection_send (chatrooms->connection, m, NULL);
 	lm_message_unref (m);
@@ -1029,28 +1049,38 @@ gossip_jabber_chatrooms_invite (GossipJabberChatrooms *chatrooms,
 void 
 gossip_jabber_chatrooms_invite_accept (GossipJabberChatrooms *chatrooms,
 				       GossipChatroomJoinCb   callback,
-				       const gchar           *nickname,
-				       const gchar           *invite_id)
+				       GossipChatroomInvite  *invite,
+				       const gchar           *nickname)
 {
 	GossipChatroom *chatroom;
+	GossipContact  *contact;
+	GossipAccount  *account;
 	gchar          *room = NULL;
+	const gchar    *id;
 	const gchar    *server;
 
-	g_return_if_fail (invite_id != NULL);
+	g_return_if_fail (chatrooms != NULL);
+	g_return_if_fail (invite != NULL);
 	g_return_if_fail (callback != NULL);
 
-	server = strstr (invite_id, "@");
+	id = gossip_chatroom_invite_get_id (invite);
+	contact = gossip_chatroom_invite_get_invitor (invite);
+
+	server = strstr (id, "@");
 
 	g_return_if_fail (server != NULL);
 	g_return_if_fail (nickname != NULL);
 	
 	if (server) {
-		room = g_strndup (invite_id, server - invite_id);
+		room = g_strndup (id, server - id);
 		server++;
 	}
 
+ 	account = gossip_contact_get_account (contact);
+
 	chatroom = g_object_new (GOSSIP_TYPE_CHATROOM, 
 				 "type", GOSSIP_CHATROOM_TYPE_NORMAL,
+				 "account", account,
 				 "server", server,
 				 "name", room,
 				 "room", room,
@@ -1066,6 +1096,44 @@ gossip_jabber_chatrooms_invite_accept (GossipJabberChatrooms *chatrooms,
 	g_free (room);
 }
 
+void 
+gossip_jabber_chatrooms_invite_decline (GossipJabberChatrooms *chatrooms,
+					GossipChatroomInvite  *invite,
+					const gchar           *reason)
+{
+	LmMessage     *m;
+	LmMessageNode *n;
+	GossipContact *own_contact;
+	GossipContact *contact;
+	const gchar   *id;
+
+	g_return_if_fail (chatrooms != NULL);
+	g_return_if_fail (invite != NULL);
+
+	own_contact = gossip_jabber_get_own_contact (chatrooms->jabber);
+	contact = gossip_chatroom_invite_get_invitor (invite);
+	id = gossip_chatroom_invite_get_id (invite);
+
+	gossip_debug (DEBUG_DOMAIN, "Invitation decline to:'%s' into room:'%s'", 
+		      gossip_contact_get_id (contact), id);
+
+	m = lm_message_new (id, LM_MESSAGE_TYPE_MESSAGE);
+	lm_message_node_set_attributes (m->node, 
+					"from", gossip_contact_get_id (own_contact), 
+					NULL);
+
+	n = lm_message_node_add_child (m->node, "x", NULL);
+	lm_message_node_set_attributes (n, "xmlns", XMPP_MUC_USER_XMLNS, NULL);
+
+	n = lm_message_node_add_child (n, "decline", NULL);
+	lm_message_node_set_attributes (n, "to", gossip_contact_get_id (contact), NULL);
+
+	n = lm_message_node_add_child (n, "reason", reason);
+
+	lm_connection_send (chatrooms->connection, m, NULL);
+	lm_message_unref (m);
+}
+
 GList * 
 gossip_jabber_chatrooms_get_rooms (GossipJabberChatrooms *chatrooms)
 {
@@ -1088,23 +1156,6 @@ jabber_chatrooms_get_rooms_foreach (gpointer               key,
 	*list = g_list_append (*list, key);
 }
 
-void
-gossip_jabber_chatrooms_set_presence (GossipJabberChatrooms  *chatrooms,
-				      GossipPresence         *presence)
-{
-	g_return_if_fail (chatrooms != NULL);
-	
-	if (chatrooms->presence) {
-		g_object_unref (chatrooms->presence);
-	}
-
-	chatrooms->presence = g_object_ref (presence);
-
-	g_hash_table_foreach (chatrooms->room_id_hash, 
-			      (GHFunc) jabber_chatrooms_set_presence_foreach,
-			      chatrooms);
-}
-
 static void
 jabber_chatrooms_set_presence_foreach (gpointer               key,
 				       JabberChatroom        *room, 
@@ -1119,7 +1170,7 @@ jabber_chatrooms_set_presence_foreach (gpointer               key,
 	connection = chatrooms->connection;
 	presence = chatrooms->presence;
 
-	show   = gossip_jabber_presence_state_to_str (presence);
+	show = gossip_jabber_presence_state_to_str (presence);
 	status = gossip_presence_get_status (presence);
 
 	m = lm_message_new_with_sub_type (gossip_jid_get_full (room->jid),
@@ -1138,6 +1189,121 @@ jabber_chatrooms_set_presence_foreach (gpointer               key,
 	lm_message_unref (m);
 }
 
+static LmMessageNode *
+jabber_chatrooms_find_muc_user_node (LmMessageNode *parent_node)
+{
+	LmMessageNode *child;
+
+	/* Should have a function in Loudmouth to find a child with xmlns */
+	child = parent_node->children;
+
+	if (!child) {
+		return NULL;
+	}
+
+	while (child) {
+		if (strcmp (child->name, "x") == 0) {
+			const gchar *xmlns;
+
+			xmlns = lm_message_node_get_attribute (child, "xmlns");
+
+			if (xmlns && strcmp (xmlns, XMPP_MUC_USER_XMLNS) == 0) {
+				return child;
+			} 
+		}
+
+		child = child->next;
+	}
+
+	return NULL;
+}
+
+static GossipChatroomRole
+jabber_chatrooms_get_role (LmMessageNode *muc_node)
+{
+	LmMessageNode *item_node;
+	const gchar   *role;
+
+	if (!muc_node) {
+		return GOSSIP_CHATROOM_ROLE_NONE;
+	}
+
+	item_node = lm_message_node_get_child (muc_node, "item");
+	if (!item_node) {
+		return GOSSIP_CHATROOM_ROLE_NONE;
+	}
+
+	role = lm_message_node_get_attribute (item_node, "role");
+	if (!role) {
+		return GOSSIP_CHATROOM_ROLE_NONE;
+	}
+
+	if (strcmp (role, "moderator") == 0) {
+		return GOSSIP_CHATROOM_ROLE_MODERATOR;
+	}
+	else if (strcmp (role, "participant") == 0) {
+		return GOSSIP_CHATROOM_ROLE_PARTICIPANT;
+	}
+	else if (strcmp (role, "visitor") == 0) {
+		return GOSSIP_CHATROOM_ROLE_VISITOR;
+	} else {
+		return GOSSIP_CHATROOM_ROLE_NONE;
+	}
+}
+
+static GossipChatroomAffiliation
+jabber_chatrooms_get_affiliation (LmMessageNode *muc_node)
+{
+	LmMessageNode *item_node;
+	const gchar   *affiliation;
+
+	if (!muc_node) {
+		return GOSSIP_CHATROOM_AFFILIATION_NONE;
+	}
+
+	item_node = lm_message_node_get_child (muc_node, "item");
+	if (!item_node) {
+		return GOSSIP_CHATROOM_AFFILIATION_NONE;
+	}
+
+	affiliation = lm_message_node_get_attribute (item_node, "affiliation");
+	if (!affiliation) {
+		return GOSSIP_CHATROOM_AFFILIATION_NONE;
+	}
+
+	if (strcmp (affiliation, "owner") == 0) {
+		return GOSSIP_CHATROOM_AFFILIATION_OWNER;
+	}
+	else if (strcmp (affiliation, "admin") == 0) {
+		return GOSSIP_CHATROOM_AFFILIATION_ADMIN;
+	}
+	else if (strcmp (affiliation, "member") == 0) {
+		return GOSSIP_CHATROOM_AFFILIATION_MEMBER;
+	} 
+	else if (strcmp (affiliation, "outcast") == 0) {
+		return GOSSIP_CHATROOM_AFFILIATION_OUTCAST;
+	} else {
+		return GOSSIP_CHATROOM_AFFILIATION_NONE;
+	}
+}
+
+void
+gossip_jabber_chatrooms_set_presence (GossipJabberChatrooms  *chatrooms,
+				      GossipPresence         *presence)
+{
+	g_return_if_fail (chatrooms != NULL);
+	
+	if (chatrooms->presence) {
+		g_object_unref (chatrooms->presence);
+	}
+
+	chatrooms->presence = g_object_ref (presence);
+
+	g_hash_table_foreach (chatrooms->room_id_hash, 
+			      (GHFunc) jabber_chatrooms_set_presence_foreach,
+			      chatrooms);
+}
+
 gboolean
 gossip_jabber_chatrooms_get_jid_is_chatroom (GossipJabberChatrooms *chatrooms,
 					     const gchar           *jid_str)
@@ -1155,11 +1321,3 @@ gossip_jabber_chatrooms_get_jid_is_chatroom (GossipJabberChatrooms *chatrooms,
 
 	return ret_val;
 }
-
-GossipContact * 
-gossip_jabber_chatrooms_get_contact (GossipJabberChatrooms *chatrooms,
-				     LmMessage             *message,
-				     gint                  *chat_id)
-{
-	return NULL;
-}
