@@ -54,23 +54,24 @@ void           gossip_jabber_chatrooms_change_nick         (GossipJabberChatroom
 							    const gchar           *new_nick);
 void           gossip_jabber_chatrooms_leave               (GossipJabberChatrooms *chatrooms,
 							    GossipChatroomId       id);
-const gchar *  gossip_jabber_chatrooms_get_room_name       (GossipJabberChatrooms *chatrooms,
+GossipChatroom *
+               gossip_jabber_chatrooms_find                (GossipJabberChatrooms *chatrooms,
 							    GossipChatroomId       id);
 void           gossip_jabber_chatrooms_invite              (GossipJabberChatrooms *chatrooms,
 							    GossipChatroomId       id,
-							    const gchar           *contact_id,
-							    const gchar           *invite);
+							    GossipContact         *contact,
+							    const gchar           *reason);
 void           gossip_jabber_chatrooms_invite_accept       (GossipJabberChatrooms *chatrooms,
 							    GossipChatroomJoinCb   callback,
-							    const gchar           *nickname,
-							    const gchar           *invite);
+							    GossipChatroomInvite  *invite,
+							    const gchar           *nickname);
+void           gossip_jabber_chatrooms_invite_decline      (GossipJabberChatrooms *chatrooms,
+							    GossipChatroomInvite  *invite,
+							    const gchar           *reason);
 GList *        gossip_jabber_chatrooms_get_rooms           (GossipJabberChatrooms *chatrooms);
 void           gossip_jabber_chatrooms_set_presence        (GossipJabberChatrooms *chatrooms,
 							    GossipPresence        *presence);
 gboolean       gossip_jabber_chatrooms_get_jid_is_chatroom (GossipJabberChatrooms *chatrooms,
 							    const gchar           *jid_str);
-GossipContact *gossip_jabber_chatrooms_get_contact         (GossipJabberChatrooms *chatrooms,
-							    LmMessage             *message,
-							    gint                  *chat_id);
 
 #endif /* __GOSSIP_JABBER_CHATROOM_H__ */
