@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2005 Imendio AB
+ * Copyright (C) 2002-2006 Imendio AB
  * Copyright (C) 2003-2004 Geert-Jan Van den Bogaerde <geertjan@gnome.org>
  * Copyright (C) 2004      Martyn Russell <mr@gnome.org>
  *
@@ -20,39 +20,50 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
 #include <string.h>
 #include <stdlib.h>
-#include <config.h>
+
 #include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
 #include <glib/gi18n.h>
 
+#include <libgossip/gossip-debug.h>
+#include <libgossip/gossip-conf.h>
+
+#include "gossip-app.h"
+#include "gossip-chat.h"
 #include "gossip-chat-window.h"
+#include "gossip-geometry.h"
 #include "gossip-marshal.h"
+#include "gossip-preferences.h"
 #include "gossip-spell.h"
 #include "gossip-spell-dialog.h"
 #include "gossip-ui-utils.h"
-#include "gossip-preferences.h"
-#include "gossip-chat.h"
-#include "gossip-app.h"
 
-#define DEBUG_MSG(x) 
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT, GossipChatPriv))
 
+#define DEBUG_DOMAIN "Chat"
+
+#define CHAT_DIR_CREATE_MODE  (S_IRUSR | S_IWUSR | S_IXUSR)
+#define CHAT_FILE_CREATE_MODE (S_IRUSR | S_IWUSR)
+
+#define MAX_INPUT_HEIGHT 150
 
 struct _GossipChatPriv {
 	GossipChatWindow *window;
 
-	GossipSpell      *spell;
-
-	/* Used to automatically shrink a window that has temporarily
-	   grown due to long input */
+	GtkTooltips      *tooltips;
+	
+	/* Used to automatically shrink a window that has temporarily grown due
+	 * to long input.
+	 */
 	gint              padding_height;
 	gint              default_window_height;
 	gint              last_input_height;
+	gboolean          vscroll_visible;
 };
 
-
 typedef struct {
 	GossipChat  *chat;
        	gchar       *word;
@@ -61,26 +72,26 @@ typedef struct {
        	GtkTextIter  end;
 } GossipChatSpell;
 
-
-static void      gossip_chat_class_init            (GossipChatClass *klass);
-static void      gossip_chat_init                  (GossipChat      *chat);
-static void      chat_finalize                     (GObject         *object);
-static void      chat_input_text_buffer_changed_cb (GtkTextBuffer   *buffer,
-						    GossipChat      *chat);
-static void      chat_text_view_size_allocate_cb   (GtkWidget       *widget,
-						    GtkAllocation   *allocation,
-						    GossipChat      *chat);
-static void      chat_text_populate_popup_cb       (GtkTextView     *view,
-						    GtkMenu         *menu,
-						    GossipChat      *chat);
-static void      chat_text_check_word_spelling_cb  (GtkMenuItem     *menuitem,
-						    GossipChatSpell *chat_spell);
-GossipChatSpell *chat_spell_new                    (GossipChat      *chat,
-						    const gchar     *word,
-						    GtkTextIter      start,
-						    GtkTextIter      end);
-static void      chat_spell_free                   (GossipChatSpell *chat_spell);
-
+static void             gossip_chat_class_init            (GossipChatClass *klass);
+static void             gossip_chat_init                  (GossipChat      *chat);
+static void             chat_finalize                     (GObject         *object);
+static void             chat_input_text_buffer_changed_cb (GtkTextBuffer   *buffer,
+							   GossipChat      *chat);
+static void             chat_text_view_scroll_hide_cb     (GtkWidget       *widget,
+							   GossipChat      *chat);
+static void             chat_text_view_size_allocate_cb   (GtkWidget       *widget,
+							   GtkAllocation   *allocation,
+							   GossipChat      *chat);
+static void             chat_text_populate_popup_cb       (GtkTextView     *view,
+							   GtkMenu         *menu,
+							   GossipChat      *chat);
+static void             chat_text_check_word_spelling_cb  (GtkMenuItem     *menuitem,
+							   GossipChatSpell *chat_spell);
+static GossipChatSpell *chat_spell_new                    (GossipChat      *chat,
+							   const gchar     *word,
+							   GtkTextIter      start,
+							   GtkTextIter      end);
+static void             chat_spell_free                   (GossipChatSpell *chat_spell);
 
 enum {
 	COMPOSING,
@@ -90,20 +101,16 @@ enum {
 	LAST_SIGNAL
 };
 
-
-static GObjectClass *parent_class = NULL;
-static guint         chat_signals[LAST_SIGNAL] = { 0 };
-
+static guint chat_signals[LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (GossipChat, gossip_chat, G_TYPE_OBJECT);
 
-
 static void
 gossip_chat_class_init (GossipChatClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GObjectClass *object_class;
 
-	parent_class = g_type_class_peek_parent (klass);
+	object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = chat_finalize;
 
@@ -115,8 +122,7 @@ gossip_chat_class_init (GossipChatClass *klass)
                               NULL, NULL,
 			      gossip_marshal_VOID__BOOLEAN,
                               G_TYPE_NONE,
-                              1,
-                              G_TYPE_BOOLEAN);
+                              1, G_TYPE_BOOLEAN);
 
 	chat_signals[NEW_MESSAGE] =
 		g_signal_new ("new-message",
@@ -124,9 +130,9 @@ gossip_chat_class_init (GossipChatClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GossipChatClass, new_message),
 			      NULL, NULL,
-			      gossip_marshal_VOID__VOID,
+			      gossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      0);
+			      1, GOSSIP_TYPE_MESSAGE);
 
 	chat_signals[NAME_CHANGED] =
 		g_signal_new ("name-changed",
@@ -147,6 +153,8 @@ gossip_chat_class_init (GossipChatClass *klass)
 			      gossip_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+
+	g_type_class_add_private (object_class, sizeof (GossipChatPriv));
 }
 
 static void
@@ -155,9 +163,6 @@ gossip_chat_init (GossipChat *chat)
 	GossipChatPriv *priv;
 	GtkTextBuffer  *buffer;
 
-	gchar          *value;
-	GList          *languages = NULL;
-
 	chat->view = gossip_chat_view_new ();
 	chat->input_text_view = gtk_text_view_new ();
 	chat->is_first_char = TRUE;
@@ -171,10 +176,12 @@ gossip_chat_init (GossipChat *chat)
 		      "wrap-mode", GTK_WRAP_WORD_CHAR,
 		      NULL);
 
-	priv = g_new0 (GossipChatPriv, 1);
-	priv->default_window_height = -1;
+	priv = GET_PRIV (chat);
 
-	chat->priv = priv;
+	priv->tooltips = gtk_tooltips_new ();
+	
+	priv->default_window_height = -1;
+	priv->vscroll_visible = FALSE;
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
 	g_signal_connect (buffer,
@@ -197,34 +204,6 @@ gossip_chat_init (GossipChat *chat)
 				    "misspelled",
 				    "underline", PANGO_UNDERLINE_ERROR,
 				    NULL);
-
-	/* get spelling languages */
-	value = gconf_client_get_string (gossip_app_get_gconf_client (),
-					 GCONF_PATH "/conversation/spell_checker_languages", 
-					 NULL);
-	
-	if (value) {
-		gchar **vlanguages;
-		gchar  *lang;
-		gint    i;
-
-		vlanguages = g_strsplit (value, ",", -1);
-		
-		for (i = 0, lang = vlanguages[i]; lang; lang = vlanguages[++i]) {
-			languages = g_list_append (languages, g_strdup (lang));
-		}
-
-		g_strfreev (vlanguages);
-	}
-	
-	g_free (value);
-	
-	priv->spell = gossip_spell_new (languages);
-	
-	if (languages) {
-		g_list_foreach (languages, (GFunc)g_free, NULL);
-		g_list_free (languages);
-	}
 }
 
 static void
@@ -233,34 +212,27 @@ chat_finalize (GObject *object)
 	GossipChat     *chat;
 	GossipChatPriv *priv;
 
-	g_return_if_fail (GOSSIP_IS_CHAT (object));
-
 	chat = GOSSIP_CHAT (object);
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 
-	if (priv->spell) {
-		gossip_spell_unref (priv->spell);
-		g_free (priv);
-	}
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (gossip_chat_parent_class)->finalize (object);
 }
 
 static void
-chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer, GossipChat *chat)
+chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer, 
+				   GossipChat    *chat)
 {
 	GossipChatPriv *priv;
-
 	GtkTextIter     start, end;
 	gchar          *str;
-	gboolean        spell_checker;
+	gboolean        spell_checker = FALSE;
 
-	priv = chat->priv;
-
- 	spell_checker = gconf_client_get_bool (gossip_app_get_gconf_client (), 
-					       "/apps/gossip/conversation/enable_spell_checker", 
-					       NULL); 
+	priv = GET_PRIV (chat);
 
+	gossip_conf_get_bool (gossip_conf_get (),
+			      GOSSIP_PREFS_CHAT_SPELL_CHECKER_ENABLED, 
+			      &spell_checker); 
+	
 	if (chat->is_first_char) {
 		GtkRequisition  req;
 		gint            window_height;
@@ -291,7 +263,7 @@ chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer, GossipChat *chat)
 		return;
 	}
 
-	if (!gossip_spell_supported () || !gossip_spell_has_backend (priv->spell)) {
+	if (!gossip_spell_supported ()) {
 		return;
 	}
 	
@@ -322,7 +294,12 @@ chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer, GossipChat *chat)
 		str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
 
 		/* spell check string */
-		correct = gossip_spell_check (priv->spell, str);
+  		if (!gossip_chat_get_is_command (str)) { 
+			correct = gossip_spell_check (str);
+ 		} else { 
+ 			correct = TRUE; 
+ 		} 
+
 		if (!correct) {
 			gtk_text_buffer_apply_tag_by_name (buffer, "misspelled", &start, &end);
 		} else {
@@ -351,26 +328,61 @@ chat_change_size_in_idle_cb (ChangeSizeData *data)
 	return FALSE;
 }
 
+static void
+chat_text_view_scroll_hide_cb (GtkWidget  *widget,
+			       GossipChat *chat)
+{
+	GossipChatPriv *priv;
+	GtkWidget      *sw;
+
+	priv = GET_PRIV (chat);
+
+	priv->vscroll_visible = FALSE;
+	g_signal_handlers_disconnect_by_func (widget, chat_text_view_scroll_hide_cb, chat);
+
+	sw = gtk_widget_get_parent (chat->input_text_view);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+					GTK_POLICY_NEVER,
+					GTK_POLICY_NEVER);
+	g_object_set (sw, "height-request", -1, NULL);
+}
+
 static void
 chat_text_view_size_allocate_cb (GtkWidget     *widget,
 		                 GtkAllocation *allocation,
 				 GossipChat    *chat)
 {
-	GossipChatPriv   *priv;
-	gint              width;
-	GtkWidget        *dialog;
-	ChangeSizeData   *data;
-	gint              window_height;
-	gint              new_height;
-	GtkAllocation    *view_allocation;
-	gint              current_height;
-	gint              diff;
+	GossipChatPriv *priv;
+	gint            width;
+	GtkWidget      *dialog;
+	ChangeSizeData *data;
+	gint            window_height;
+	gint            new_height;
+	GtkAllocation  *view_allocation;
+	gint            current_height;
+	gint            diff;
+	GtkWidget      *sw;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 
 	if (priv->default_window_height <= 0) {
 		return;
 	}
+	
+	sw = gtk_widget_get_parent (widget);
+	if (sw->allocation.height >= MAX_INPUT_HEIGHT && !priv->vscroll_visible) {
+		GtkWidget *vscroll;
+
+		priv->vscroll_visible = TRUE;
+		gtk_widget_set_size_request (sw, -1, MAX_INPUT_HEIGHT);
+		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+						GTK_POLICY_NEVER,
+						GTK_POLICY_AUTOMATIC);
+		vscroll = gtk_scrolled_window_get_vscrollbar (GTK_SCROLLED_WINDOW (sw));
+		g_signal_connect (vscroll, "hide",
+				  G_CALLBACK (chat_text_view_scroll_hide_cb),
+				  chat);
+	}
 
 	if (priv->last_input_height <= allocation->height) {
 		priv->last_input_height = allocation->height;
@@ -410,24 +422,59 @@ chat_text_view_size_allocate_cb (GtkWidget     *widget,
 			 data, g_free);
 }
 
+static void
+chat_insert_smiley_activate_cb (GtkWidget  *menuitem, 
+				GossipChat *chat)
+{
+	GtkTextBuffer *buffer;
+	GtkTextIter    iter;
+	const gchar   *smiley;
+
+	smiley = g_object_get_data (G_OBJECT (menuitem), "smiley_text");
+
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
+	gtk_text_buffer_get_end_iter (buffer, &iter);
+	gtk_text_buffer_insert (buffer, &iter, 
+				smiley, -1);
+}
+
 static void
 chat_text_populate_popup_cb (GtkTextView *view,
 			     GtkMenu     *menu,
 			     GossipChat  *chat)
 {
-	GtkTextBuffer      *buffer;
-	GtkTextTagTable    *table;
-	GtkTextTag         *tag;
-	gint                x, y;
-	GtkTextIter         iter, start, end;
-	GtkWidget          *item;
-	gchar              *str = NULL;
-	GossipChatSpell    *chat_spell;
+	GossipChatPriv  *priv;
+	GtkTextBuffer   *buffer;
+	GtkTextTagTable *table;
+	GtkTextTag      *tag;
+	gint             x, y;
+	GtkTextIter      iter, start, end;
+	GtkWidget       *item;
+	gchar           *str = NULL;
+	GossipChatSpell *chat_spell;
+	GtkWidget       *smiley_menu;
 
+	priv = GET_PRIV (chat);
+	
+	/* Add the emoticon menu. */
+	item = gtk_separator_menu_item_new ();
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+	gtk_widget_show (item);
+	
+	item = gtk_menu_item_new_with_mnemonic (_("Insert Smiley"));
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+	gtk_widget_show (item);
+
+	smiley_menu = gossip_chat_view_get_smiley_menu (
+		G_CALLBACK (chat_insert_smiley_activate_cb), 
+		chat,
+		priv->tooltips);
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), smiley_menu);
+
+	/* Add the spell check menu item. */
 	buffer = gtk_text_view_get_buffer (view);
 	table = gtk_text_buffer_get_tag_table (buffer);
-	
-	/* handle misspelled tags */
+
 	tag = gtk_text_tag_table_lookup (table, "misspelled");
 	
 	gtk_widget_get_pointer (GTK_WIDGET (view), &x, &y);
@@ -471,7 +518,17 @@ chat_text_populate_popup_cb (GtkTextView *view,
 	gtk_widget_show (item);
 }
 
-GossipChatSpell *
+static void     
+chat_text_check_word_spelling_cb (GtkMenuItem     *menuitem, 
+				  GossipChatSpell *chat_spell)
+{
+	gossip_spell_dialog_show (chat_spell->chat,
+				  chat_spell->start,
+				  chat_spell->end,
+				  chat_spell->word);
+}
+
+static GossipChatSpell *
 chat_spell_new (GossipChat  *chat, 
 		const gchar *word,
 		GtkTextIter  start,
@@ -479,16 +536,10 @@ chat_spell_new (GossipChat  *chat,
 {
 	GossipChatSpell *chat_spell;
 
-	g_return_val_if_fail (chat != NULL, NULL);
-	g_return_val_if_fail (word != NULL, NULL);
-
 	chat_spell = g_new0 (GossipChatSpell, 1);
 
 	chat_spell->chat = g_object_ref (chat);
-
-
 	chat_spell->word = g_strdup (word);
-
 	chat_spell->start = start;
 	chat_spell->end = end;
 	
@@ -503,19 +554,26 @@ chat_spell_free (GossipChatSpell *chat_spell)
 	g_free (chat_spell);
 }
 
-static void     
-chat_text_check_word_spelling_cb (GtkMenuItem     *menuitem, 
-				  GossipChatSpell *chat_spell)
+gboolean
+gossip_chat_get_is_command (const gchar *str)
 {
-	GossipChatPriv *priv;
+	g_return_val_if_fail (str != NULL, FALSE);
 
-	priv = chat_spell->chat->priv;
+	if (str[0] != '/') {
+		return FALSE;
+	}
+	
+	if (g_str_has_prefix (str, "/me")) {
+		return TRUE;
+	}
+	else if (g_str_has_prefix (str, "/nick")) {
+		return TRUE;
+	}
+	else if (g_str_has_prefix (str, "/topic")) {
+		return TRUE;
+	}
 
-	gossip_spell_dialog_show (chat_spell->chat,
-				  priv->spell,
-				  chat_spell->start,
-				  chat_spell->end,
-				  chat_spell->word);
+	return FALSE;
 }
 
 void
@@ -528,14 +586,13 @@ gossip_chat_correct_word (GossipChat  *chat,
 
 	g_return_if_fail (chat != NULL);
 	g_return_if_fail (new_word != NULL);
-	g_return_if_fail (strlen (new_word) > 0);
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
 
 	gtk_text_buffer_delete (buffer, &start, &end);
 	gtk_text_buffer_insert (buffer, &start, 
 				new_word, 
-				g_utf8_strlen (new_word, -1));
+				-1);
 }
 
 const gchar *
@@ -598,16 +655,16 @@ gossip_chat_get_own_contact (GossipChat *chat)
 	return NULL;
 }
 
-void
-gossip_chat_get_geometry (GossipChat *chat,
-		          int        *width,
-			  int        *height)
+GossipChatroom *
+gossip_chat_get_chatroom (GossipChat *chat)
 {
-	g_return_if_fail (GOSSIP_IS_CHAT (chat));
+	g_return_val_if_fail (GOSSIP_IS_CHAT (chat), NULL);
 
-	if (GOSSIP_CHAT_GET_CLASS (chat)->get_geometry) {
-		GOSSIP_CHAT_GET_CLASS (chat)->get_geometry (chat, width, height);
+	if (GOSSIP_CHAT_GET_CLASS (chat)->get_chatroom) {
+		return GOSSIP_CHAT_GET_CLASS (chat)->get_chatroom (chat);
 	}
+
+	return NULL;
 }
 
 GtkWidget *
@@ -623,12 +680,12 @@ gossip_chat_get_widget (GossipChat *chat)
 }
 
 gboolean
-gossip_chat_get_group_chat (GossipChat *chat)
+gossip_chat_is_group_chat (GossipChat *chat)
 {
 	g_return_val_if_fail (GOSSIP_IS_CHAT (chat), FALSE);
 
-	if (GOSSIP_CHAT_GET_CLASS (chat)->get_group_chat) {
-		return GOSSIP_CHAT_GET_CLASS (chat)->get_group_chat (chat);
+	if (GOSSIP_CHAT_GET_CLASS (chat)->is_group_chat) {
+		return GOSSIP_CHAT_GET_CLASS (chat)->is_group_chat (chat);
 	}
 
 	return FALSE;
@@ -657,6 +714,26 @@ gossip_chat_set_show_contacts (GossipChat *chat,
 	}
 }
 
+void 
+gossip_chat_save_geometry (GossipChat *chat, 
+			   gint        x,
+			   gint        y,
+			   gint        w,
+			   gint        h)
+{
+	gossip_geometry_save_for_chat (chat, x, y, w, h);
+}
+
+void 
+gossip_chat_load_geometry (GossipChat *chat,
+			   gint       *x,
+			   gint       *y,
+			   gint       *w,
+			   gint       *h)
+{
+	gossip_geometry_load_for_chat (chat, x, y, w, h);
+}
+
 void
 gossip_chat_clear (GossipChat *chat)
 {
@@ -666,15 +743,23 @@ gossip_chat_clear (GossipChat *chat)
 }
 
 void
-gossip_chat_set_window (GossipChat *chat, GossipChatWindow *window)
+gossip_chat_set_window (GossipChat       *chat, 
+			GossipChatWindow *window)
 {
-	chat->priv->window = window;
+	GossipChatPriv *priv;
+
+	priv = GET_PRIV (chat);
+	priv->window = window;
 }
 
 GossipChatWindow *
 gossip_chat_get_window (GossipChat *chat)
 {
-	return chat->priv->window;
+	GossipChatPriv *priv;
+
+	priv = GET_PRIV (chat);
+
+	return priv->window;
 }
 
 void
@@ -688,7 +773,7 @@ gossip_chat_scroll_down (GossipChat *chat)
 void
 gossip_chat_cut (GossipChat *chat)
 {
-	GtkTextBuffer  *buffer;
+	GtkTextBuffer *buffer;
 	
 	g_return_if_fail (GOSSIP_IS_CHAT (chat));
 
@@ -705,7 +790,7 @@ gossip_chat_cut (GossipChat *chat)
 void
 gossip_chat_copy (GossipChat *chat)
 {
-	GtkTextBuffer         *buffer;
+	GtkTextBuffer *buffer;
 	
 	g_return_if_fail (GOSSIP_IS_CHAT (chat));
 
@@ -727,8 +812,8 @@ gossip_chat_copy (GossipChat *chat)
 void
 gossip_chat_paste (GossipChat *chat)
 {
-	GtkTextBuffer         *buffer;
-	GtkClipboard          *clipboard;
+	GtkTextBuffer *buffer;
+	GtkClipboard  *clipboard;
 
 	g_return_if_fail (GOSSIP_IS_CHAT (chat));
 
@@ -745,13 +830,13 @@ gossip_chat_present (GossipChat *chat)
 
 	g_return_if_fail (GOSSIP_IS_CHAT (chat));
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
 	if (priv->window == NULL) {
 		GossipChatWindow *window;
 		gboolean          for_group_chat;
 
-		for_group_chat = gossip_chat_get_group_chat (chat);
+		for_group_chat = gossip_chat_is_group_chat (chat);
 
 		/* get the default window for either group chats or
 		   normal chats, we do not want to mix them */
@@ -799,12 +884,12 @@ gossip_chat_should_highlight_nick (GossipMessage *message,
 	gchar         *ch;
 	gboolean       ret_val;
 
-	DEBUG_MSG (("Chat: Highlighting nickname"));
+	g_return_val_if_fail (GOSSIP_IS_MESSAGE (message), FALSE);
+
+	gossip_debug (DEBUG_DOMAIN, "Highlighting nickname");
 
 	ret_val = FALSE;
 
-	g_return_val_if_fail (GOSSIP_IS_MESSAGE (message), FALSE);
-
 	msg = gossip_message_get_body (message);
 	if (!msg) {
 		return FALSE;
@@ -838,8 +923,8 @@ gossip_chat_should_highlight_nick (GossipMessage *message,
 		}
 	}
 
-	ch = ch + g_utf8_strlen (cf_to, -1);
-	if (ch >= cf_msg + g_utf8_strlen (cf_msg, -1)) {
+	ch = ch + strlen (cf_to);
+	if (ch >= cf_msg + strlen (cf_msg)) {
 		ret_val = TRUE;
 		goto finished;
 	}
