@@ -18,21 +18,26 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+#include <string.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/libgnomeui.h>
-#include <libgnomevfs/gnome-vfs.h>
+#include <gtk/gtk.h>
 
+#ifdef HAVE_GNOME
+#include <libgnomevfs/gnome-vfs.h>
+#endif
+
+#include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-event-manager.h>
 #include <libgossip/gossip-ft.h>
 #include <libgossip/gossip-protocol.h>
 #include <libgossip/gossip-utils.h>
 
 #include "gossip-ft-window.h"
+#include "gossip-ui-utils.h"
 #include "gossip-app.h"
 
-#define DEBUG_MSG(x)  
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");  */
-
+#define DEBUG_DOMAIN "FileTransferWindow"
 
 typedef struct {
 	GossipProtocol *protocol;
@@ -40,6 +45,7 @@ typedef struct {
 	GossipVCard    *vcard;
 } FTData;
 
+#ifdef HAVE_GNOME
 
 static void ft_window_protocol_connected_cb    (GossipSession      *session,
 						GossipAccount      *account,
@@ -48,6 +54,7 @@ static void ft_window_protocol_connected_cb    (GossipSession      *session,
 static void ft_window_protocol_disconnected_cb (GossipSession      *session,
 						GossipAccount      *account,
 						GossipProtocol     *protocol,
+						gint                reason,
 						gpointer            user_data);
 static void ft_window_request_cb               (GossipProtocol     *protocol,
 						GossipFT           *ft,
@@ -70,10 +77,11 @@ static void ft_window_filechooser_response_cb  (GtkDialog          *dialog,
 						gint                response_id,
 						GossipContact      *contact);
 
-
 void
 gossip_ft_window_init (GossipSession *session)
 {
+ 	g_object_ref (session);
+	
 	g_signal_connect (session, 
 			  "protocol-connected",
 			  G_CALLBACK (ft_window_protocol_connected_cb),
@@ -94,6 +102,8 @@ gossip_ft_window_finalize (GossipSession *session)
  	g_signal_handlers_disconnect_by_func (session, 
 					      ft_window_protocol_disconnected_cb, 
 					      NULL);
+
+	g_object_unref (session);
 }
 
 /*
@@ -121,6 +131,7 @@ static void
 ft_window_protocol_disconnected_cb (GossipSession  *session,
 				    GossipAccount  *account,
 				    GossipProtocol *protocol,
+				    gint            reason,
 				    gpointer        user_data)
 {
  	g_signal_handlers_disconnect_by_func (protocol, 
@@ -154,7 +165,7 @@ ft_window_request_cb (GossipProtocol *protocol,
 
 	gossip_event_manager_add (gossip_app_get_event_manager (),
 				  event, 
-				  (GossipEventActivatedFunction)ft_window_event_activated_cb,
+				  (GossipEventActivateFunction) ft_window_event_activated_cb,
 				  G_OBJECT (protocol));
 }
 
@@ -256,7 +267,7 @@ ft_window_vcard_cb (GossipResult  result,
 
 	contact = gossip_ft_get_contact (data->ft);
 
-	gossip_glade_get_file_simple (GLADEDIR "/file-transfer.glade",
+	gossip_glade_get_file_simple ("file-transfer.glade",
 				      "file_transfer_dialog",
 				      NULL,
 				      "file_transfer_dialog", &dialog,
@@ -305,7 +316,7 @@ ft_window_vcard_cb (GossipResult  result,
 		GtkWidget *href;
 		GtkWidget *alignment;
 
-		href = gnome_href_new (url, url);
+		href = gossip_link_button_new (url, url);
 
 		alignment = gtk_alignment_new (0, 1, 0, 0.5);
 		gtk_container_add (GTK_CONTAINER (alignment), href);
@@ -400,7 +411,7 @@ ft_window_filechooser_create (GossipContact *contact)
 	GtkWidget     *dialog;
 	GtkFileFilter *filter;
 
-	DEBUG_MSG (("FileTransferWindow: Creating filechooser..."));
+	gossip_debug (DEBUG_DOMAIN, "Creating filechooser...");
 
 	dialog = g_object_new (GTK_TYPE_FILE_CHOOSER_DIALOG,
 			       "action", GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -442,14 +453,14 @@ ft_window_filechooser_response_cb (GtkDialog     *dialog,
 		if (list) {
 			GSList *l;
 			
-			DEBUG_MSG (("FileTransferWindow: File chooser selected files:"));
+			gossip_debug (DEBUG_DOMAIN, "File chooser selected files:");
 			
 			for (l = list; l; l = l->next) {
 				gchar *file;
 				
 				file = l->data;
 				
-				DEBUG_MSG (("FileTransferWindow: \t%s", file));
+				gossip_debug (DEBUG_DOMAIN, "\t%s", file);
 				gossip_ft_window_send_file_from_uri (contact, file);
 				
 				g_free (file);
@@ -457,10 +468,35 @@ ft_window_filechooser_response_cb (GtkDialog     *dialog,
 			
 			g_slist_free (list);
 		} else {
-			DEBUG_MSG (("FileTransferWindow: File chooser had no files selected"));
+			gossip_debug (DEBUG_DOMAIN, "File chooser had no files selected");
 		}
 	}
 
 	g_object_unref (contact);
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
+
+#else
+
+void
+gossip_ft_window_init               (GossipSession *session)
+{
+}
+
+void
+gossip_ft_window_finalize           (GossipSession *session)
+{
+}
+
+void
+gossip_ft_window_send_file          (GossipContact *account)
+{
+}
+
+void
+gossip_ft_window_send_file_from_uri (GossipContact *contact,
+				     const gchar   *file)
+{
+}
+
+#endif
