@@ -35,6 +35,7 @@
 #define DEBUG_DOMAIN "Notify"
 
 #define NOTIFY_MESSAGE_TIME 20000
+#define NOTIFY_SUBSCRIPTION_TIME 20000
 
 /* Max length of the body part of a message we show in the
  * notification. 
@@ -46,38 +47,44 @@
  */ 
 #define NOTIFY_WAIT_TIME 10000
 
-static const gchar *       notify_get_status_from_presence    (GossipPresence     *presence);
-static gboolean            notify_get_is_busy                 (void);
-static void                notify_contact_online              (GossipContact      *contact);
-static void                notify_online_action_cb            (NotifyNotification *notify,
-							       gchar              *label,
-							       GossipContact      *contact);
-static void                notify_new_message_contact_cb      (NotifyNotification *notify,
-							       gchar              *label,
-							       GossipEventManager *event_manager);
-static NotifyNotification *notify_new_message                 (GossipEventManager *event_manager,
-							       GossipMessage      *message);
-static gboolean            notify_protocol_timeout_cb         (GossipAccount      *account);
-static void                notify_protocol_connected_cb       (GossipSession      *session,
-							       GossipAccount      *account,
-							       GossipProtocol     *protocol,
-							       gpointer            user_data);
-static void                notify_protocol_disconnected_cb    (GossipSession      *session,
-							       GossipAccount      *account,
-							       GossipProtocol     *protocol,
-							       gpointer            user_data);
-static void                notify_contact_presence_updated_cb (GossipSession      *session,
-							       GossipContact      *contact,
-							       gpointer            user_data);
-static void                notify_event_added_cb              (GossipEventManager *event_manager,
-							       GossipEvent        *event,
-							       gpointer            user_data);
-static gboolean            notify_event_remove_foreach        (gpointer            key,
-							       GossipEvent        *event,
-							       GossipEvent        *event_to_compare);
-static void                notify_event_removed_cb            (GossipEventManager *event_manager,
-							       GossipEvent        *event,
-							       gpointer            user_data);
+static const gchar *       notify_get_status_from_presence        (GossipPresence     *presence);
+static gboolean            notify_get_is_busy                     (void);
+static void                notify_contact_online                  (GossipContact      *contact);
+static void                notify_online_action_cb                (NotifyNotification *notify,
+								   gchar              *label,
+								   GossipContact      *contact);
+static void                notify_subscription_request_default_cb (NotifyNotification *notify,
+								   gchar              *label,
+								   gpointer            user_data);
+static void                notify_subscription_request_show       (GossipContact      *contact);
+static void                notify_new_message_contact_cb          (NotifyNotification *notify,
+								   gchar              *label,
+								   GossipEventManager *event_manager);
+static NotifyNotification *notify_new_message                     (GossipEventManager *event_manager,
+								   GossipMessage      *message);
+static gboolean            notify_protocol_timeout_cb             (GossipAccount      *account);
+static void                notify_protocol_connected_cb           (GossipSession      *session,
+								   GossipAccount      *account,
+								   GossipProtocol     *protocol,
+								   gpointer            user_data);
+static void                notify_protocol_disconnected_cb        (GossipSession      *session,
+								   GossipAccount      *account,
+								   GossipProtocol     *protocol,
+								   gint                reason,
+								   gpointer            user_data);
+static void                notify_contact_presence_updated_cb     (GossipSession      *session,
+								   GossipContact      *contact,
+								   gpointer            user_data);
+static gboolean            notify_subscription_request_show_cb    (GossipContact      *contact);
+static void                notify_event_added_cb                  (GossipEventManager *event_manager,
+								   GossipEvent        *event,
+								   gpointer            user_data);
+static gboolean            notify_event_remove_foreach            (gpointer            key,
+								   GossipEvent        *event,
+								   GossipEvent        *event_to_compare);
+static void                notify_event_removed_cb                (GossipEventManager *event_manager,
+								   GossipEvent        *event,
+								   gpointer            user_data);
 
 enum {
 	NOTIFY_SHOW_MESSAGE,
@@ -91,7 +98,6 @@ static GHashTable *message_notifications = NULL;
 static GHashTable *event_notifications = NULL;
 static GtkWidget  *attach_widget = NULL;
 
-
 static const gchar *
 notify_get_status_from_presence (GossipPresence *presence)
 {
@@ -153,11 +159,12 @@ notify_contact_online (GossipContact *contact)
 {
 	GossipPresence     *presence;
 	NotifyNotification *notify;
-	GdkPixbuf          *pixbuf;
+	GdkPixbuf          *pixbuf = NULL;
 	gchar              *title;
 	const gchar        *status;
 	GError             *error = NULL;
 	gboolean            show_popup;
+	gboolean            show_avatars;
 
 	if (!gossip_conf_get_bool (gossip_conf_get (),
 				   GOSSIP_PREFS_POPUPS_WHEN_AVAILABLE,
@@ -177,7 +184,20 @@ notify_contact_online (GossipContact *contact)
 		      gossip_contact_get_id (contact));
 
 	presence = gossip_contact_get_active_presence (contact);
-	pixbuf = gossip_pixbuf_for_presence (presence);
+
+	show_avatars = FALSE;
+	gossip_conf_get_bool (gossip_conf_get (),
+			       GOSSIP_PREFS_UI_SHOW_AVATARS,
+			       &show_avatars);
+
+	if (show_avatars) {
+		pixbuf = gossip_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
+	}
+
+	if (!pixbuf) {
+		/* Fall back to presence state */
+		pixbuf = gossip_pixbuf_for_presence (presence); 
+	}
 
 	title = g_markup_printf_escaped (_("%s has come online"), 
 					 gossip_contact_get_name (contact));
@@ -216,6 +236,77 @@ notify_contact_online (GossipContact *contact)
 	}
 }
 
+static void
+notify_subscription_request_default_cb (NotifyNotification *notify,
+					gchar              *label,
+					gpointer            user_data)
+{
+	/* FIXME: Show subscription dialog */
+}
+
+static void
+notify_subscription_request_show (GossipContact *contact)
+{
+	NotifyNotification  *notify;
+	GdkPixbuf           *pixbuf = NULL;
+	GError              *error = NULL;
+	const gchar         *name;
+	gchar               *message;
+	gboolean             show_avatars = FALSE;
+
+	name = gossip_contact_get_name (contact);
+	if (name) {
+		message = g_strdup_printf (_("%s wants to be added to your contact list."), name);
+	} else {
+		message = g_strdup (_("Someone wants to be added to your contact list."));
+	}
+
+	gossip_conf_get_bool (gossip_conf_get (),
+			      GOSSIP_PREFS_UI_SHOW_AVATARS,
+			      &show_avatars);
+
+	if (show_avatars) {
+		pixbuf = gossip_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
+	}
+
+	if (!pixbuf) {
+		/* Fall back to presence state */
+		pixbuf = gossip_pixbuf_from_stock (GTK_STOCK_DIALOG_QUESTION, 
+						   GTK_ICON_SIZE_MENU);
+	}
+
+	notify = notify_notification_new (_("Subscription request"),
+					  message,
+					  NULL,
+					  attach_widget);
+	g_free (message);
+
+	notify_notification_set_urgency (notify, NOTIFY_URGENCY_NORMAL);
+	notify_notification_set_timeout (notify, NOTIFY_SUBSCRIPTION_TIME);
+
+	if (pixbuf) {
+		notify_notification_set_icon_from_pixbuf (notify, pixbuf);
+		g_object_unref (pixbuf);
+	}
+
+	notify_notification_add_action (notify, "default", _("Default"),
+					(NotifyActionCallback) notify_subscription_request_default_cb,
+					NULL, NULL);
+
+	g_signal_connect (notify,
+			  "closed",
+			  G_CALLBACK (notify_closed_cb),
+			  NULL);
+
+	if (!notify_notification_show (notify, &error)) {
+		g_warning ("Failed to send notification: %s",
+			   error->message);
+		g_error_free (error);
+	}
+
+	return;
+}
+
 static void
 notify_new_message_default_cb (NotifyNotification *notify,
 			       gchar              *label,
@@ -252,10 +343,13 @@ notify_new_message_contact_cb (NotifyNotification *notify,
 
 	event = g_hash_table_lookup (event_notifications, notify);
 	if (event) {
+		GtkWindow *parent;
+
 		message = GOSSIP_MESSAGE (gossip_event_get_data (event));
 		contact = gossip_message_get_sender (message);
 
-		gossip_contact_info_dialog_show (contact);
+		parent = GTK_WINDOW (gossip_app_get_window ());
+		gossip_contact_info_dialog_show (contact, parent);
 
 		g_hash_table_remove (event_notifications, notify);
 		g_hash_table_remove (message_notifications, contact);
@@ -272,13 +366,14 @@ notify_new_message (GossipEventManager *event_manager,
 { 
 	GossipContact      *contact;
 	NotifyNotification *notify;
-	GdkPixbuf          *pixbuf;
+	GdkPixbuf          *pixbuf = NULL;
+	GError             *error = NULL;
 	gchar              *title;
 	gchar              *body_copy;
 	const gchar        *body_stripped;
 	gchar              *str;
 	gint                len;
-	GError             *error = NULL;
+	gboolean            show_avatars;
 
 	contact = gossip_message_get_sender (message);
 
@@ -296,7 +391,19 @@ notify_new_message (GossipEventManager *event_manager,
 	gossip_debug (DEBUG_DOMAIN, "Setting up notification for new message from:'%s'", 
 		      gossip_contact_get_id (contact));
 
-	pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE, GTK_ICON_SIZE_MENU);
+	show_avatars = FALSE;
+	gossip_conf_get_bool (gossip_conf_get (),
+			       GOSSIP_PREFS_UI_SHOW_AVATARS,
+			       &show_avatars);
+
+	if (show_avatars) {
+		pixbuf = gossip_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
+	}
+
+	if (!pixbuf) {
+		/* Fall back to message icon */
+		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE, GTK_ICON_SIZE_MENU);
+	}
 
 	title = g_strdup_printf (_("New message from %s"), 
 				 gossip_contact_get_name (contact));
@@ -332,6 +439,7 @@ notify_new_message (GossipEventManager *event_manager,
 	notify_notification_add_action (notify, "respond", _("Show"),
 					(NotifyActionCallback) notify_new_message_default_cb,
 					g_object_ref (event_manager), NULL);
+
 	if (gossip_contact_get_type (contact) == GOSSIP_CONTACT_TYPE_TEMPORARY) {
 		notify_notification_add_action (
 			notify, "contact", _("Contact Information"),
@@ -400,6 +508,7 @@ static void
 notify_protocol_disconnected_cb (GossipSession  *session,
 				 GossipAccount  *account,
 				 GossipProtocol *protocol,
+				 gint            reason,
 				 gpointer        user_data)
 {
 	g_hash_table_remove (account_states, account);
@@ -475,6 +584,14 @@ notify_contact_presence_updated_cb (GossipSession *session,
 	}
 }
 
+static gboolean
+notify_subscription_request_show_cb (GossipContact *contact)
+{
+	notify_subscription_request_show (contact);
+
+	return FALSE;
+}
+
 static void
 notify_event_added_cb (GossipEventManager *event_manager,
 		       GossipEvent        *event,
@@ -488,7 +605,7 @@ notify_event_added_cb (GossipEventManager *event_manager,
 		NotifyNotification  *notify;
 		GossipMessage       *message;
 		GossipContact       *contact;
-		
+
 		message = GOSSIP_MESSAGE (gossip_event_get_data (event));
 		contact = gossip_message_get_sender (message);
 
@@ -508,6 +625,12 @@ notify_event_added_cb (GossipEventManager *event_manager,
 			}
 		}
 	}
+	else if (type == GOSSIP_EVENT_SUBSCRIPTION_REQUEST) {
+		/* Give a chance to the server to have time to give us infos */
+		g_timeout_add (1000,
+			       (GSourceFunc) notify_subscription_request_show_cb,
+			       gossip_event_get_data (event));
+	}
 }
 
 static gboolean
@@ -563,6 +686,65 @@ gossip_notify_set_attach_widget (GtkWidget *new_attach_widget)
 	}
 }
 
+static void
+notify_hint_closed_cb (NotifyNotification *notification,
+		       gpointer            user_data)
+{
+	GFunc        func;
+	const gchar *conf_path;
+
+	conf_path = g_object_get_data (G_OBJECT (notification), "conf_path");
+	func = g_object_get_data (G_OBJECT (notification), "func");
+
+	gossip_conf_set_bool (gossip_conf_get (), conf_path, FALSE);
+
+	g_object_unref (notification);
+
+	if (func) {
+		(func) ((gpointer) conf_path, user_data);
+	}
+}
+
+gboolean
+gossip_notify_hint_show (const gchar        *conf_path, 
+			 const gchar        *summary,
+			 const gchar        *message,
+			 GFunc               func,
+			 gpointer            user_data)
+{
+	gboolean            ok;
+	gboolean            show_hint = TRUE;
+	NotifyNotification *notify;
+	GError             *error = NULL;
+
+	g_return_val_if_fail (conf_path != NULL, FALSE);
+	g_return_val_if_fail (summary != NULL, FALSE);
+
+	ok = gossip_conf_get_bool (gossip_conf_get (), 
+				   conf_path, 
+				   &show_hint);
+	
+	if (ok && !show_hint) {
+		return FALSE;
+	}
+
+	notify = notify_notification_new (summary, message, NULL, attach_widget);
+	g_object_set_data_full (G_OBJECT (notify), "conf_path", g_strdup (conf_path), g_free);
+	g_object_set_data (G_OBJECT (notify), "func", func);
+	g_signal_connect (notify,
+			  "closed",
+			  G_CALLBACK (notify_hint_closed_cb),
+			  user_data);
+
+	if (!notify_notification_show (notify, &error)) {
+		g_warning ("Failed to send notification: %s",
+			   error->message);
+		g_error_free (error);
+	}
+
+	return TRUE;
+}
+
 void
 gossip_notify_init (GossipSession      *session,
 		    GossipEventManager *event_manager)
