@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2004 Imendio AB
+ * Copyright (C) 2003-2006 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,52 +25,37 @@
 #include <glib/gi18n.h>
 
 #include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-conf.h>
 
 #include "gossip-app.h"
 #include "gossip-stock.h"
 #include "gossip-ui-utils.h"
 #include "gossip-preferences.h"
+#include "gossip-theme-manager.h"
 #include "gossip-spell.h"
 
-
 typedef struct {
-	GossipApp *app;
- 	
 	GtkWidget *dialog;
+
 	GtkWidget *notebook;
-	GtkWidget *sound_checkbutton;
-	GtkWidget *silent_busy_checkbutton;
-	GtkWidget *silent_away_checkbutton;
-	GtkWidget *smileys_checkbutton;
-        GtkWidget *separate_windows_checkbutton;
-	GtkWidget *compact_checkbutton;
-	GtkWidget *leaving_entry;
-	GtkWidget *away_entry;
-	GtkWidget *spell_checker_vbox;
-	GtkWidget *spell_checker_checkbutton;
-	GtkWidget *spell_checker_treeview;
 
-	GtkWidget *theme_combobox;
-	
-	GList     *ids;
+	GtkWidget *checkbutton_show_avatars;
+	GtkWidget *checkbutton_show_smileys;
+	GtkWidget *combobox_chat_theme;
+	GtkWidget *checkbutton_theme_chat_room;
+        GtkWidget *checkbutton_separate_chat_windows;
+
+	GtkWidget *checkbutton_sounds_for_messages;
+	GtkWidget *checkbutton_sounds_when_busy;
+	GtkWidget *checkbutton_sounds_when_away;
+	GtkWidget *checkbutton_popups_when_available;
+
+	GtkWidget *treeview_spell_checker;
+        GtkWidget *checkbutton_spell_checker;
+
+	GList     *notify_ids;
 } GossipPreferences;
 
-
-enum {
-	COL_LANG_ENABLED,
-	COL_LANG_CODE,
-	COL_LANG_NAME,
-	COL_LANG_COUNT
-};
-
-enum {
-	COL_COMBO_VISIBLE_NAME,
-	COL_COMBO_NAME,
-	COL_COMBO_COUNT
-};
-
-
-
 static void     preferences_setup_widgets                (GossipPreferences      *preferences);
 static void     preferences_languages_setup              (GossipPreferences      *preferences);
 static void     preferences_languages_add                (GossipPreferences      *preferences);
@@ -88,33 +73,28 @@ static void     preferences_languages_cell_toggled_cb    (GtkCellRendererToggle
 							  gchar                  *path_string,
 							  GossipPreferences      *preferences);
 static void     preferences_themes_setup                 (GossipPreferences      *preferences);
-static void     preferences_set_boolean_from_gconf       (const gchar            *key,
+static void     preferences_widget_sync_bool             (const gchar            *key,
 							  GtkWidget              *widget);
-static void     preferences_set_int_from_gconf           (const gchar            *key,
+static void     preferences_widget_sync_int              (const gchar            *key,
 							  GtkWidget              *widget);
-static void     preferences_set_string_from_gconf        (const gchar            *key,
+static void     preferences_widget_sync_string           (const gchar            *key,
 							  GtkWidget              *widget);
-static void     preferences_set_string_combo_from_gconf  (const gchar            *key,
+static void     preferences_widget_sync_string_combo     (const gchar            *key,
 							  GtkWidget              *widget);
-static void     preferences_notify_int_cb                (GConfClient            *client,
-							  guint                   cnxn_id,
-							  GConfEntry             *entry,
+static void     preferences_notify_int_cb                (GossipConf             *conf,
+							  const gchar            *key,
 							  gpointer                user_data);
-static void     preferences_notify_string_cb             (GConfClient            *client,
-							  guint                   cnxn_id,
-							  GConfEntry             *entry,
+static void     preferences_notify_string_cb             (GossipConf             *conf,
+							  const gchar            *key,
 							  gpointer                user_data);
-static void     preferences_notify_string_combo_cb       (GConfClient            *client,
-							  guint                   cnxn_id,
-							  GConfEntry             *entry,
+static void     preferences_notify_string_combo_cb       (GossipConf             *conf,
+							  const gchar            *key,
 							  gpointer                user_data);
-static void     preferences_notify_bool_cb               (GConfClient            *client,
-							  guint                   cnxn_id,
-							  GConfEntry             *entry,
+static void     preferences_notify_bool_cb               (GossipConf             *conf,
+							  const gchar            *key,
 							  gpointer                user_data);
-static void     preferences_notify_sensitivity_cb        (GConfClient            *client,
-							  guint                   cnxn_id,
-							  GConfEntry             *entry,
+static void     preferences_notify_sensitivity_cb        (GossipConf             *conf,
+							  const gchar            *key,
 							  gpointer                user_data);
 static void     preferences_hookup_spin_button           (GossipPreferences      *preferences,
 							  const gchar            *key,
@@ -145,42 +125,68 @@ static void     preferences_response_cb                  (GtkWidget
 							  gint                    response,
 							  GossipPreferences      *preferences);
 
+enum {
+	COL_LANG_ENABLED,
+	COL_LANG_CODE,
+	COL_LANG_NAME,
+	COL_LANG_COUNT
+};
 
+enum {
+	COL_COMBO_VISIBLE_NAME,
+	COL_COMBO_NAME,
+	COL_COMBO_COUNT
+};
 
 static void
 preferences_setup_widgets (GossipPreferences *preferences)
 {
 	preferences_hookup_toggle_button (preferences,
-					  GCONF_PATH "/sound/play_sounds",
-					  preferences->sound_checkbutton);
+					  GOSSIP_PREFS_SOUNDS_FOR_MESSAGES,
+					  preferences->checkbutton_sounds_for_messages);
 	preferences_hookup_toggle_button (preferences,
-					  GCONF_PATH "/sound/silent_away",
-					  preferences->silent_away_checkbutton);
+					  GOSSIP_PREFS_SOUNDS_WHEN_AWAY,
+					  preferences->checkbutton_sounds_when_away);
 	preferences_hookup_toggle_button (preferences,
-					  GCONF_PATH "/sound/silent_busy",
-					  preferences->silent_busy_checkbutton);
+					  GOSSIP_PREFS_SOUNDS_WHEN_BUSY,
+					  preferences->checkbutton_sounds_when_busy);
+	preferences_hookup_toggle_button (preferences,
+					  GOSSIP_PREFS_POPUPS_WHEN_AVAILABLE,
+					  preferences->checkbutton_popups_when_available);
+
 	preferences_hookup_sensitivity (preferences,
-					GCONF_PATH "/sound/play_sounds",
-					preferences->silent_away_checkbutton);
+					GOSSIP_PREFS_SOUNDS_FOR_MESSAGES,
+					preferences->checkbutton_sounds_when_away);
 	preferences_hookup_sensitivity (preferences,
-					GCONF_PATH "/sound/play_sounds",
-					preferences->silent_busy_checkbutton);
+					GOSSIP_PREFS_SOUNDS_FOR_MESSAGES,
+					preferences->checkbutton_sounds_when_busy);
 
         preferences_hookup_toggle_button (preferences,
-					  GCONF_PATH "/conversation/graphical_smileys",
-					  preferences->smileys_checkbutton);
-
-        preferences_hookup_toggle_button (preferences,
-					  GCONF_PATH "/ui/separate_chat_windows",
-					  preferences->separate_windows_checkbutton);
+					  GOSSIP_PREFS_UI_SEPARATE_CHAT_WINDOWS,
+					  preferences->checkbutton_separate_chat_windows);
 
 	preferences_hookup_toggle_button (preferences,
-					  GCONF_PATH "/conversation/enable_spell_checker",
-					  preferences->spell_checker_checkbutton);
+					  GOSSIP_PREFS_UI_SHOW_AVATARS,
+					  preferences->checkbutton_show_avatars);
+
+        preferences_hookup_toggle_button (preferences,
+					  GOSSIP_PREFS_CHAT_SHOW_SMILEYS,
+					  preferences->checkbutton_show_smileys);
 
 	preferences_hookup_string_combo (preferences,
-					 GCONF_PATH "/conversation/theme",
-					 preferences->theme_combobox);
+					 GOSSIP_PREFS_CHAT_THEME,
+					 preferences->combobox_chat_theme);
+
+        preferences_hookup_toggle_button (preferences,
+					  GOSSIP_PREFS_CHAT_THEME_CHAT_ROOM,
+					  preferences->checkbutton_theme_chat_room);
+
+	preferences_hookup_toggle_button (preferences,
+					  GOSSIP_PREFS_CHAT_SPELL_CHECKER_ENABLED,
+					  preferences->checkbutton_spell_checker);
+	preferences_hookup_sensitivity (preferences,
+					GOSSIP_PREFS_CHAT_SPELL_CHECKER_ENABLED,
+					preferences->treeview_spell_checker);
 }
 
 static void 
@@ -194,7 +200,7 @@ preferences_languages_setup (GossipPreferences *preferences)
 	GtkCellRenderer   *renderer;
 	guint              col_offset;
 
-	view = GTK_TREE_VIEW (preferences->spell_checker_treeview);
+	view = GTK_TREE_VIEW (preferences->treeview_spell_checker);
 
 	store = gtk_list_store_new (COL_LANG_COUNT,
 				    G_TYPE_BOOLEAN,  /* enabled */
@@ -242,40 +248,31 @@ preferences_languages_add (GossipPreferences *preferences)
 {
 	GtkTreeView  *view;
 	GtkListStore *store;
-	GossipSpell  *spell;
 	GList        *codes, *l;
 
-	view = GTK_TREE_VIEW (preferences->spell_checker_treeview);
+	view = GTK_TREE_VIEW (preferences->treeview_spell_checker);
 	store = GTK_LIST_STORE (gtk_tree_view_get_model (view));
 
-	spell = gossip_spell_new (NULL);
-	codes = gossip_spell_get_language_codes (spell);
-
+	codes = gossip_spell_get_language_codes ();
 	for (l = codes; l; l = l->next) {
 		GtkTreeIter  iter;
 		const gchar *code;
-		const gchar *language;
+		const gchar *name;
 
 		code = l->data; 
-		language = gossip_spell_get_language_name (spell, code);
-		
-		if (!language || strlen (language) < 1) {
+		name = gossip_spell_get_language_name (code);
+		if (!name) {
 			continue;
 		}
 
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter, 
 				    COL_LANG_CODE, code,
-				    COL_LANG_NAME, language,
+				    COL_LANG_NAME, name,
 				    -1);
 	}
 
-	g_list_foreach (codes, (GFunc)g_free, NULL);
-	g_list_free (codes);
-
-	if (spell) {
-		gossip_spell_unref (spell);
-	}
+	gossip_spell_free_language_codes (codes);
 }
 
 static void
@@ -286,7 +283,7 @@ preferences_languages_save (GossipPreferences *preferences)
 
 	gchar             *languages = NULL;
 
-	view = GTK_TREE_VIEW (preferences->spell_checker_treeview);
+	view = GTK_TREE_VIEW (preferences->treeview_spell_checker);
 	model = gtk_tree_view_get_model (view);
 	
 	gtk_tree_model_foreach (model,
@@ -294,14 +291,13 @@ preferences_languages_save (GossipPreferences *preferences)
 				&languages);
 
 	if (!languages) {
-		/* default to english */
+		/* Default to english */
 		languages = g_strdup ("en");
 	}
 	
-	gconf_client_set_string (gossip_app_get_gconf_client (),
-				 GCONF_PATH "/conversation/spell_checker_languages", 
-				 languages,
-				 NULL);
+	gossip_conf_set_string (gossip_conf_get (),
+				 GOSSIP_PREFS_CHAT_SPELL_CHECKER_LANGUAGES,
+				 languages);
 
 	g_free (languages);
 }
@@ -345,25 +341,21 @@ preferences_languages_save_foreach (GtkTreeModel  *model,
 static void 
 preferences_languages_load (GossipPreferences *preferences)
 {
-	GtkTreeView    *view;
-	GtkTreeModel   *model;
+	GtkTreeView   *view;
+	GtkTreeModel  *model;
+	gchar         *value;
+	gchar        **vlanguages;
 
-	gchar          *value;
-	gchar         **vlanguages;
-
-	value = gconf_client_get_string (
-		gossip_app_get_gconf_client (),
-		GCONF_PATH "/conversation/spell_checker_languages", 
-		NULL);
-	
-	if (!value) {
+	if (!gossip_conf_get_string (gossip_conf_get (),
+				      GOSSIP_PREFS_CHAT_SPELL_CHECKER_LANGUAGES,
+				      &value) || !value) {
 		return;
 	}
 
 	vlanguages = g_strsplit (value, ",", -1);
 	g_free (value);
 
-	view = GTK_TREE_VIEW (preferences->spell_checker_treeview);
+	view = GTK_TREE_VIEW (preferences->treeview_spell_checker);
 	model = gtk_tree_view_get_model (view);
 
 	gtk_tree_model_foreach (model,
@@ -388,7 +380,6 @@ preferences_languages_load_foreach (GtkTreeModel  *model,
 		return TRUE;
 	}
 
-
 	gtk_tree_model_get (model, iter, COL_LANG_CODE, &code, -1);
 	if (!code) {
 		return FALSE;
@@ -416,7 +407,7 @@ preferences_languages_cell_toggled_cb (GtkCellRendererToggle *cell,
 	GtkTreeIter   iter;
 	gboolean      enabled;
 
-	view = GTK_TREE_VIEW (preferences->spell_checker_treeview);
+	view = GTK_TREE_VIEW (preferences->treeview_spell_checker);
 	model = gtk_tree_view_get_model (view);
 	store = GTK_LIST_STORE (model);
 	
@@ -436,73 +427,73 @@ preferences_languages_cell_toggled_cb (GtkCellRendererToggle *cell,
 static void
 preferences_themes_setup (GossipPreferences *preferences)
 {
-	GtkComboBox     *combo;
-	GtkListStore    *model;
-	GtkTreeIter      iter;
+	GtkComboBox   *combo;
+	GtkListStore  *model;
+	GtkTreeIter    iter;
+	const gchar  **themes;
+	gint           i;
 
-	combo = GTK_COMBO_BOX (preferences->theme_combobox);
+	combo = GTK_COMBO_BOX (preferences->combobox_chat_theme);
 
 	model = gtk_list_store_new (COL_COMBO_COUNT,
 				    G_TYPE_STRING,
 				    G_TYPE_STRING);
 
-	gtk_list_store_append (model, &iter);
-	gtk_list_store_set (model, &iter,
-			    COL_COMBO_VISIBLE_NAME, _("Classic"),
-			    COL_COMBO_NAME, "classic",
-			    -1);
-	
-	gtk_list_store_append (model, &iter);
-	gtk_list_store_set (model, &iter,
-			    COL_COMBO_VISIBLE_NAME, _("Blue"),
-			    COL_COMBO_NAME, "blue",
-			    -1);
+	themes = gossip_theme_manager_get_themes ();
+ 	for (i = 0; themes[i]; i += 2) {
+		gtk_list_store_append (model, &iter);
+		gtk_list_store_set (model, &iter,
+				    COL_COMBO_VISIBLE_NAME, _(themes[i + 1]),
+				    COL_COMBO_NAME, themes[i],
+				    -1);
+	}
 	
 	gtk_combo_box_set_model (combo, GTK_TREE_MODEL (model));
 	g_object_unref (model);
 }	
 
 static void
-preferences_set_boolean_from_gconf (const gchar *key, GtkWidget *widget)
+preferences_widget_sync_bool (const gchar *key, GtkWidget *widget)
 {
 	gboolean value;
 	
-	value = gconf_client_get_bool (gossip_app_get_gconf_client (), key, NULL);
-	
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), value);
+	if (gossip_conf_get_bool (gossip_conf_get (), key, &value)) {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), value);
+	}
 }
 
 static void
-preferences_set_int_from_gconf (const gchar *key, GtkWidget *widget)
+preferences_widget_sync_int (const gchar *key, GtkWidget *widget)
 {
 	gint value;
 	
-	value = gconf_client_get_int (gossip_app_get_gconf_client (), key, NULL);
-	
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), value);
+	if (gossip_conf_get_int (gossip_conf_get (), key, &value)) {
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), value);
+	}
 }
 
 static void
-preferences_set_string_from_gconf (const gchar *key, GtkWidget *widget)
+preferences_widget_sync_string (const gchar *key, GtkWidget *widget)
 {
 	gchar *value;
 	
-	value = gconf_client_get_string (gossip_app_get_gconf_client (), key, NULL);
-	
-	gtk_entry_set_text (GTK_ENTRY (widget), value);
-
-	g_free (value);
+	if (gossip_conf_get_string (gossip_conf_get (), key, &value) && value) {
+		gtk_entry_set_text (GTK_ENTRY (widget), value);
+		g_free (value);
+	}
 }
 
 static void
-preferences_set_string_combo_from_gconf (const gchar *key, GtkWidget *widget)
+preferences_widget_sync_string_combo (const gchar *key, GtkWidget *widget)
 {
 	gchar        *value;
 	GtkTreeModel *model; 
 	GtkTreeIter   iter;
 	gboolean      found;
 	
-	value = gconf_client_get_string (gossip_app_get_gconf_client (), key, NULL);
+	if (!gossip_conf_get_string (gossip_conf_get (), key, &value)) {
+		return;
+	}
 
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
 
@@ -527,7 +518,7 @@ preferences_set_string_combo_from_gconf (const gchar *key, GtkWidget *widget)
 		} while (gtk_tree_model_iter_next (model, &iter));
 	}
 
-	/* Fallback to classic. */
+	/* Fallback to the first one. */
 	if (!found) {
 		if (gtk_tree_model_get_iter_first (model, &iter)) {
 			gtk_combo_box_set_active_iter (GTK_COMBO_BOX (widget), &iter);
@@ -538,74 +529,75 @@ preferences_set_string_combo_from_gconf (const gchar *key, GtkWidget *widget)
 }
 
 static void
-preferences_notify_int_cb (GConfClient *client,
-			   guint        cnxn_id,
-			   GConfEntry  *entry,
-			      gpointer   user_data)
+preferences_notify_int_cb (GossipConf  *conf,
+			   const gchar *key,
+			   gpointer     user_data)
 {
-	GConfValue *value;
-
-	value = gconf_entry_get_value (entry);
+	gint value;
 	
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (user_data),
-				   gconf_value_get_int (value));
+	if (gossip_conf_get_int (conf, key, &value)) {
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (user_data), value);
+	}
 }
 
 static void
-preferences_notify_string_cb (GConfClient *client,
-		 guint        cnxn_id,
-		 GConfEntry  *entry,
-		 gpointer     user_data)
+preferences_notify_string_cb (GossipConf  *conf,
+			      const gchar *key,
+			      gpointer     user_data)
 {
-	GConfValue *value;
+	gchar *value;
 	
-	value = gconf_entry_get_value (entry);
-
-	gtk_entry_set_text (GTK_ENTRY (user_data),
-			    gconf_value_get_string (value));
+	if (gossip_conf_get_string (conf, key, &value) && value) {
+		gtk_entry_set_text (GTK_ENTRY (user_data), value);
+		g_free (value);
+	}
 }
 
 static void
-preferences_notify_string_combo_cb (GConfClient *client,
-				    guint        cnxn_id,
-				    GConfEntry  *entry,
+preferences_notify_string_combo_cb (GossipConf  *conf,
+				    const gchar *key,
 				    gpointer     user_data)
 {
-	preferences_set_string_combo_from_gconf (gconf_entry_get_key (entry), user_data);
+	preferences_widget_sync_string_combo (key, user_data);
 }
 
 static void
-preferences_notify_bool_cb (GConfClient *client,
-			    guint        cnxn_id,
-			    GConfEntry  *entry,
+preferences_notify_bool_cb (GossipConf  *conf,
+			    const gchar *key,
 			    gpointer     user_data)
 {
-	GConfValue *value;
-	
-	value = gconf_entry_get_value (entry);
-
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (user_data),
-				      gconf_value_get_bool (value));
+	preferences_widget_sync_bool (key, user_data);
+/*	
+	if (gossip_conf_get_bool (conf, key, &value)) {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (user_data),
+					      gconf_value_get_bool (value));
+					      }
+*/
 }
 
 static void
-preferences_notify_sensitivity_cb (GConfClient *client,
-				   guint        cnxn_id,
-				   GConfEntry  *entry,
+preferences_notify_sensitivity_cb (GossipConf  *conf,
+				   const gchar *key,
 				   gpointer     user_data)
 {
-	GConfValue *value;
+	gboolean value;
 	
-	value = gconf_entry_get_value (entry);
+	if (gossip_conf_get_bool (conf, key, &value)) {
+		gtk_widget_set_sensitive (GTK_WIDGET (user_data), value);
+	}
+}
 
-	gtk_widget_set_sensitive (GTK_WIDGET (user_data),
-				  gconf_value_get_bool (value));
+static void
+preferences_add_id (GossipPreferences *preferences, guint id)
+{
+	preferences->notify_ids = g_list_prepend (preferences->notify_ids,
+						  GUINT_TO_POINTER (id));
 }
 
 static void
 preferences_hookup_spin_button (GossipPreferences *preferences,
-		    const gchar       *key,
-		    GtkWidget         *widget)
+				const gchar       *key,
+				GtkWidget         *widget)
 {
 	guint id;
 
@@ -614,7 +606,7 @@ preferences_hookup_spin_button (GossipPreferences *preferences,
 		preferences_hookup_spin_button (preferences, key, widget);
 	}
 	
-	preferences_set_int_from_gconf (key, widget);
+	preferences_widget_sync_int (key, widget);
 
 	g_object_set_data_full (G_OBJECT (widget), "key",
 				g_strdup (key), g_free); 
@@ -624,21 +616,19 @@ preferences_hookup_spin_button (GossipPreferences *preferences,
 			  G_CALLBACK (preferences_spin_button_value_changed_cb),
 			  NULL);
 
-	id = gconf_client_notify_add (gossip_app_get_gconf_client (),
+	id = gossip_conf_notify_add (gossip_conf_get (),
 				      key,
 				      preferences_notify_int_cb,
-				      widget,
-				      NULL,
-				      NULL);
-	
-	preferences->ids = g_list_prepend (preferences->ids,
-					   GUINT_TO_POINTER (id));
+				      widget);
+	if (id) {
+		preferences_add_id (preferences, id);
+	}
 }
 
 static void
 preferences_hookup_entry (GossipPreferences *preferences,
-	      const gchar       *key,
-	      GtkWidget         *widget)
+			  const gchar       *key,
+			  GtkWidget         *widget)
 {
 	guint id;
 
@@ -646,7 +636,7 @@ preferences_hookup_entry (GossipPreferences *preferences,
 		preferences_hookup_entry (preferences, key, widget);
 	}
 	
-	preferences_set_string_from_gconf (key, widget);
+	preferences_widget_sync_string (key, widget);
 
 	g_object_set_data_full (G_OBJECT (widget), "key",
 				g_strdup (key), g_free); 
@@ -656,25 +646,23 @@ preferences_hookup_entry (GossipPreferences *preferences,
 			  G_CALLBACK (preferences_entry_value_changed_cb),
 			  NULL);
 
-	id = gconf_client_notify_add (gossip_app_get_gconf_client (),
+	id = gossip_conf_notify_add (gossip_conf_get (),
 				      key,
 				      preferences_notify_string_cb,
-				      widget,
-				      NULL,
-				      NULL);
-	
-	preferences->ids = g_list_prepend (preferences->ids,
-					   GUINT_TO_POINTER (id));
+				      widget);
+	if (id) {
+		preferences_add_id (preferences, id);
+	}
 }
 
 static void
 preferences_hookup_toggle_button (GossipPreferences *preferences,
-		      const gchar       *key,
-		      GtkWidget         *widget)
+				  const gchar       *key,
+				  GtkWidget         *widget)
 {
 	guint id;
 	
-	preferences_set_boolean_from_gconf (key, widget);
+	preferences_widget_sync_bool (key, widget);
 	
 	g_object_set_data_full (G_OBJECT (widget), "key",
 				g_strdup (key), g_free); 
@@ -684,15 +672,13 @@ preferences_hookup_toggle_button (GossipPreferences *preferences,
 			  G_CALLBACK (preferences_toggle_button_toggled_cb),
 			  NULL);
 
-	id = gconf_client_notify_add (gossip_app_get_gconf_client (),
+	id = gossip_conf_notify_add (gossip_conf_get (),
 				      key,
 				      preferences_notify_bool_cb,
-				      widget,
-				      NULL,
-				      NULL);
-	
-	preferences->ids = g_list_prepend (preferences->ids,
-					   GUINT_TO_POINTER (id));
+				      widget);
+	if (id) {
+		preferences_add_id (preferences, id);
+	}
 }
 
 static void
@@ -702,7 +688,7 @@ preferences_hookup_string_combo (GossipPreferences *preferences,
 {
 	guint id;
 
-	preferences_set_string_combo_from_gconf (key, widget);
+	preferences_widget_sync_string_combo (key, widget);
 	
 	g_object_set_data_full (G_OBJECT (widget), "key",
 				g_strdup (key), g_free); 
@@ -712,37 +698,34 @@ preferences_hookup_string_combo (GossipPreferences *preferences,
 			  G_CALLBACK (preferences_string_combo_changed_cb),
 			  NULL);
 
-	id = gconf_client_notify_add (gossip_app_get_gconf_client (),
+	id = gossip_conf_notify_add (gossip_conf_get (),
 				      key,
 				      preferences_notify_string_combo_cb,
-				      widget,
-				      NULL,
-				      NULL);
-	
-	preferences->ids = g_list_prepend (preferences->ids,
-					   GUINT_TO_POINTER (id));
+				      widget);
+	if (id) {
+		preferences_add_id (preferences, id);
+	}
 }
 
 static void
 preferences_hookup_sensitivity (GossipPreferences *preferences,
-			       const gchar       *key,
-			       GtkWidget         *widget)
+				const gchar       *key,
+				GtkWidget         *widget)
 {
 	gboolean value;
 	guint    id;
 
-	value = gconf_client_get_bool (gossip_app_get_gconf_client (), key, NULL);
-	gtk_widget_set_sensitive (widget, value);
+	if (gossip_conf_get_bool (gossip_conf_get (), key, &value)) {
+		gtk_widget_set_sensitive (widget, value);
+	}
 	
-	id = gconf_client_notify_add (gossip_app_get_gconf_client (),
+	id = gossip_conf_notify_add (gossip_conf_get (),
 				      key,
 				      preferences_notify_sensitivity_cb,
-				      widget,
-				      NULL,
-				      NULL);
-
-	preferences->ids = g_list_prepend (preferences->ids,
-					   GUINT_TO_POINTER (id));
+				      widget);
+	if (id) {
+		preferences_add_id (preferences, id);
+	}
 }
 
 static void
@@ -753,10 +736,9 @@ preferences_spin_button_value_changed_cb (GtkWidget *button,
 
 	key = g_object_get_data (G_OBJECT (button), "key");
 
-	gconf_client_set_int (gossip_app_get_gconf_client (),
+	gossip_conf_set_int (gossip_conf_get (),
 			      key,
-			      gtk_spin_button_get_value (GTK_SPIN_BUTTON (button)),
-			      NULL);
+			      gtk_spin_button_get_value (GTK_SPIN_BUTTON (button)));
 }
 
 static void
@@ -767,10 +749,9 @@ preferences_entry_value_changed_cb (GtkWidget *entry,
 
 	key = g_object_get_data (G_OBJECT (entry), "key");
 	
-	gconf_client_set_string (gossip_app_get_gconf_client (),
+	gossip_conf_set_string (gossip_conf_get (),
 				 key,
-				 gtk_entry_get_text (GTK_ENTRY (entry)),
-				 NULL);
+				 gtk_entry_get_text (GTK_ENTRY (entry)));
 }
 
 static void
@@ -781,10 +762,9 @@ preferences_toggle_button_toggled_cb (GtkWidget *button,
 
 	key = g_object_get_data (G_OBJECT (button), "key");
 
-	gconf_client_set_bool (gossip_app_get_gconf_client (),
+	gossip_conf_set_bool (gossip_conf_get (),
 			       key,
-			       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)),
-			       NULL);
+			       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
 }
 
 static void
@@ -804,32 +784,11 @@ preferences_string_combo_changed_cb (GtkWidget *combo,
 		gtk_tree_model_get (model, &iter,
 				    COL_COMBO_NAME, &name,
 				    -1);
-		
-		gconf_client_set_string (gossip_app_get_gconf_client (),
-					 key,
-					 name,
-					 NULL);
-
+		gossip_conf_set_string (gossip_conf_get (), key, name);
 		g_free (name);
 	}
 }
 
-static void
-preferences_destroy_cb (GtkWidget         *widget,
-			GossipPreferences *preferences)
-{
-	GList *l;
-
-	for (l = preferences->ids; l; l = l->next) {
-		guint id = GPOINTER_TO_UINT (l->data);
-
-		gconf_client_notify_remove (gossip_app_get_gconf_client (), id);
-	}
-
-	g_list_free (preferences->ids);
-	g_free (preferences);
-}
-
 static void
 preferences_response_cb (GtkWidget         *widget,
 			 gint               response,
@@ -838,6 +797,22 @@ preferences_response_cb (GtkWidget         *widget,
 	gtk_widget_destroy (widget);
 }
 
+static void
+preferences_destroy_cb (GtkWidget         *widget,
+			GossipPreferences *preferences)
+{
+	GList *l;
+
+	for (l = preferences->notify_ids; l; l = l->next) {
+		guint id;
+
+		id = GPOINTER_TO_UINT (l->data);
+		gossip_conf_notify_remove (gossip_conf_get (), id);
+	}
+
+	g_list_free (preferences->notify_ids);
+	g_free (preferences);
+}
 
 void
 gossip_preferences_show (void)
@@ -851,27 +826,35 @@ gossip_preferences_show (void)
 	}
 	
         preferences = g_new0 (GossipPreferences, 1);
-	preferences->app = gossip_app_get ();
 
 	glade = gossip_glade_get_file (
-		GLADEDIR "/main.glade",
+		"main.glade",
 		"preferences_dialog",
 		NULL,
 		"preferences_dialog", &preferences->dialog,
 		"notebook", &preferences->notebook,
-		"sound_checkbutton", &preferences->sound_checkbutton,
-		"silent_busy_checkbutton", &preferences->silent_busy_checkbutton,
-		"silent_away_checkbutton", &preferences->silent_away_checkbutton,
-		"smileys_checkbutton", &preferences->smileys_checkbutton,
-		"separate_windows_checkbutton", &preferences->separate_windows_checkbutton,
-		"spell_checker_vbox", &preferences->spell_checker_vbox,
-		"spell_checker_checkbutton", &preferences->spell_checker_checkbutton,
-		"spell_checker_treeview", &preferences->spell_checker_treeview,
-		"theme_combobox", &preferences->theme_combobox,
+		"checkbutton_show_avatars", &preferences->checkbutton_show_avatars,
+		"checkbutton_show_smileys", &preferences->checkbutton_show_smileys,
+		"combobox_chat_theme", &preferences->combobox_chat_theme,
+		"checkbutton_theme_chat_room", &preferences->checkbutton_theme_chat_room,
+		"checkbutton_separate_chat_windows", &preferences->checkbutton_separate_chat_windows,
+		"checkbutton_sounds_for_messages", &preferences->checkbutton_sounds_for_messages,
+		"checkbutton_sounds_when_busy", &preferences->checkbutton_sounds_when_busy,
+		"checkbutton_sounds_when_away", &preferences->checkbutton_sounds_when_away,
+		"checkbutton_popups_when_available", &preferences->checkbutton_popups_when_available,
+		"treeview_spell_checker", &preferences->treeview_spell_checker,
+		"checkbutton_spell_checker", &preferences->checkbutton_spell_checker,
 		NULL);
 
-	g_object_add_weak_pointer (G_OBJECT (preferences->dialog),
-				   (gpointer) &preferences);
+	gossip_glade_connect (glade, 
+			      preferences,
+			      "preferences_dialog", "destroy", preferences_destroy_cb,
+			      "preferences_dialog", "response", preferences_response_cb,
+			      NULL);
+
+	g_object_unref (glade);
+
+	g_object_add_weak_pointer (G_OBJECT (preferences->dialog), (gpointer) &preferences);
 	
 	preferences_themes_setup (preferences);
 
@@ -881,21 +864,15 @@ gossip_preferences_show (void)
 	preferences_languages_add (preferences);
 	preferences_languages_load (preferences);
 	
-	gossip_glade_connect (glade, 
-			      preferences,
-			      "preferences_dialog", "destroy", preferences_destroy_cb,
-			      "preferences_dialog", "response", preferences_response_cb,
-			      NULL);
-	
 	if (gossip_spell_supported ()) {
-		GtkWidget *spell_checking_page;
+		GtkWidget *page;
 
-		spell_checking_page = gtk_notebook_get_nth_page (
-			GTK_NOTEBOOK (preferences->notebook), 1);
-		gtk_widget_show (spell_checking_page);
+		page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (preferences->notebook), 2);
+		gtk_widget_show (page);
 	}
 
 	gtk_window_set_transient_for (GTK_WINDOW (preferences->dialog),
 				      GTK_WINDOW (gossip_app_get_window ()));
+
 	gtk_widget_show (preferences->dialog);
 }
