@@ -26,6 +26,7 @@
 
 #include "gossip-app.h"
 #include "gossip-group-chat.h"
+
 #include <libgossip/gossip-message.h>
 
 G_BEGIN_DECLS
@@ -44,8 +45,7 @@ typedef struct _GossipPrivateChatPriv GossipPrivateChatPriv;
 #include "gossip-chat-window.h"
 
 struct _GossipPrivateChat {
-	GossipChat             parent;
-        GossipPrivateChatPriv *priv;
+	GossipChat parent;
 };
 
 struct _GossipPrivateChatClass {
@@ -53,7 +53,8 @@ struct _GossipPrivateChatClass {
 };
 
 GType               gossip_private_chat_get_type           (void);
-GossipPrivateChat * gossip_private_chat_new                (GossipContact     *contact);
+GossipPrivateChat * gossip_private_chat_new                (GossipContact     *own_contact,
+							    GossipContact     *contact);
 GossipPrivateChat * gossip_private_chat_get_for_group_chat (GossipContact     *contact,
 						            GossipGroupChat   *g_chat);
 void                gossip_private_chat_append_message     (GossipPrivateChat *chat,
