@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Imendio HB
+ * Copyright (C) 2003 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,14 +21,27 @@
 #ifndef __GOSSIP_STOCK_H__
 #define __GOSSIP_STOCK_H__
 
-#define GOSSIP_STOCK_OFFLINE    "gossip-offline"
-#define GOSSIP_STOCK_AVAILABLE  "gossip-available"
-#define GOSSIP_STOCK_BUSY       "gossip-busy"
-#define GOSSIP_STOCK_AWAY       "gossip-away"
-#define GOSSIP_STOCK_EXT_AWAY   "gossip-extended-away"
-#define GOSSIP_STOCK_MESSAGE    "gossip-message"
-#define GOSSIP_STOCK_TYPING     "gossip-typing"
+#define GOSSIP_STOCK_OFFLINE             "gossip-offline"
+#define GOSSIP_STOCK_AVAILABLE           "gossip-available"
+#define GOSSIP_STOCK_BUSY                "gossip-busy"
+#define GOSSIP_STOCK_AWAY                "gossip-away"
+#define GOSSIP_STOCK_EXT_AWAY            "gossip-extended-away"
+#define GOSSIP_STOCK_PENDING             "gossip-pending"
 
-void gossip_stock_init (void);
+#define GOSSIP_STOCK_MESSAGE             "gossip-message"
+#define GOSSIP_STOCK_TYPING              "gossip-typing"
+
+
+#define GOSSIP_STOCK_CONTACT_INFORMATION "vcard_16"
+
+#define GOSSIP_STOCK_AIM                 "gossip-aim"
+#define GOSSIP_STOCK_ICQ                 "gossip-icq"
+#define GOSSIP_STOCK_MSN                 "gossip-msn"
+#define GOSSIP_STOCK_YAHOO               "gossip-yahoo"
+
+#define GOSSIP_STOCK_GROUP_MESSAGE       "gossip-group-message"
+
+void gossip_stock_init     (void);
+void gossip_stock_finalize (void);
 
 #endif /* __GOSSIP_STOCK_ICONS_H__ */
