@@ -19,22 +19,23 @@
  */
 
 #include <config.h>
-
 #include <string.h>
 #include <unistd.h>
-
 #include <gtk/gtk.h> 
 #include <glade/glade.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-href.h>
+
+#ifdef HAVE_GNOME
 #include <libgnomeui/libgnomeui.h>  
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
-#include <loudmouth/loudmouth.h>
+#endif
 
-#include <libgossip/gossip-session.h>
+#include <libgossip/gossip-debug.h>
+#include <libgossip/gossip-conf.h>
 #include <libgossip/gossip-protocol.h>
-#include <libgossip/gossip-vcard.h>
+#include <libgossip/gossip-session.h>
 #include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-vcard.h>
 
 #include "gossip-account-chooser.h"
 #include "gossip-app.h"
@@ -42,8 +43,7 @@
 #include "gossip-image-chooser.h"
 #include "gossip-preferences.h"
 
-#define DEBUG_MSG(x)
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");  */
+#define DEBUG_DOMAIN "VCardDialog"
 
 #define STRING_EMPTY(x) ((x) == NULL || (x)[0] == '\0')
 
@@ -84,45 +84,45 @@ typedef struct {
 	GtkWidget *avatar_chooser;
 
 	guint      timeout_id; 
-	guint      saved_id;
-
 	gboolean   requesting_vcard;
-
 	gint       last_account_selected;
 
+#ifdef HAVE_GNOME
 	GnomeThumbnailFactory *thumbs;
+#endif
 } GossipVCardDialog;
 
-static void     vcard_dialog_avatar_chooser_response_cb (GtkWidget         *widget,
-							 gint               response,
-							 GossipVCardDialog *dialog);
-static void     vcard_dialog_avatar_clicked_cb          (GtkWidget         *button,
-							 GossipVCardDialog *dialog);
-static void     vcard_dialog_avatar_update_preview_cb   (GtkFileChooser    *chooser,
-							 GossipVCardDialog *dialog);
-static void     vcard_dialog_set_account_to_last        (GossipVCardDialog *dialog);
-static void     vcard_dialog_lookup_start               (GossipVCardDialog *dialog);
-static void     vcard_dialog_lookup_stop                (GossipVCardDialog *dialog);
-static void     vcard_dialog_get_vcard_cb               (GossipResult       result,
-							 GossipVCard       *vcard,
-							 GossipVCardDialog *dialog);
-static void     vcard_dialog_set_vcard                  (GossipVCardDialog *dialog);
-static void     vcard_dialog_set_vcard_cb               (GossipResult       result,
-							 GossipVCardDialog *dialog);
-static gboolean vcard_dialog_timeout_cb                 (GossipVCardDialog *dialog);
-static void     vcard_dialog_account_changed_cb         (GtkWidget         *combo_box,
-							 GossipVCardDialog *dialog);
-static void     vcard_dialog_response_cb                (GtkDialog         *widget,
-							 gint               response,
-							 GossipVCardDialog *dialog);
-static void     vcard_dialog_destroy_cb                 (GtkWidget         *widget,
-							 GossipVCardDialog *dialog);
+static GtkWidget *vcard_dialog_create_avatar_chooser      (void);
+static void       vcard_dialog_avatar_chooser_response_cb (GtkWidget         *widget,
+							   gint               response,
+							   GossipVCardDialog *dialog);
+static void       vcard_dialog_avatar_clicked_cb          (GtkWidget         *button,
+							   GossipVCardDialog *dialog);
+static void       vcard_dialog_avatar_update_preview_cb   (GtkFileChooser    *chooser,
+							   GossipVCardDialog *dialog);
+static void       vcard_dialog_set_account_to_last        (GossipVCardDialog *dialog);
+static void       vcard_dialog_lookup_start               (GossipVCardDialog *dialog);
+static void       vcard_dialog_lookup_stop                (GossipVCardDialog *dialog);
+static void       vcard_dialog_get_vcard_cb               (GossipResult       result,
+							   GossipVCard       *vcard,
+							   GossipVCardDialog *dialog);
+static void       vcard_dialog_set_vcard                  (GossipVCardDialog *dialog);
+static void       vcard_dialog_set_vcard_cb               (GossipResult       result,
+							   gpointer           user_data);
+static gboolean   vcard_dialog_timeout_cb                 (GossipVCardDialog *dialog);
+static void       vcard_dialog_account_changed_cb         (GtkWidget         *combo_box,
+							   GossipVCardDialog *dialog);
+static void       vcard_dialog_response_cb                (GtkDialog         *widget,
+							   gint               response,
+							   GossipVCardDialog *dialog);
+static void       vcard_dialog_destroy_cb                 (GtkWidget         *widget,
+							   GossipVCardDialog *dialog);
 
-/* Called from Glade, so it shouldn't be static. */
-GtkWidget *vcard_dialog_create_avatar_chooser (gpointer data);
 
-GtkWidget *
-vcard_dialog_create_avatar_chooser (gpointer data)
+static GossipVCardDialog *dialog = NULL;
+
+static GtkWidget *
+vcard_dialog_create_avatar_chooser (void)
 {
 	GtkWidget *chooser;
 
@@ -132,12 +132,12 @@ vcard_dialog_create_avatar_chooser (gpointer data)
 						 AVATAR_MAX_WIDTH, 
 						 AVATAR_MAX_HEIGHT);
 
-	gtk_widget_show_all (chooser);
-
 	gtk_widget_set_size_request (chooser, 
 				     AVATAR_MAX_WIDTH / 2,
 				     AVATAR_MAX_HEIGHT / 2);
 
+	gtk_widget_show_all (chooser);
+
 	return chooser;
 }
 
@@ -151,21 +151,20 @@ vcard_dialog_avatar_chooser_response_cb (GtkWidget         *widget,
 		gchar *path;
 
 		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (widget));
-		gossip_image_chooser_set_from_file (GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser),
-						    filename);
+		gossip_image_chooser_set_from_file 
+			(GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser), filename);
 		g_free (filename);
 
 		path = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (widget));
 		if (path) {
-			gconf_client_set_string (gossip_app_get_gconf_client (),
-						 GCONF_UI_AVATAR_DIRECTORY,
-						 path,
-						 NULL);
+			gossip_conf_set_string (gossip_conf_get (),
+						 GOSSIP_PREFS_UI_AVATAR_DIRECTORY,
+						 path);
 			g_free (path);
 		}
 	} else if (response == GTK_RESPONSE_NO) {
-		gossip_image_chooser_set_image_data (GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser),
-						     NULL, 0);
+		gossip_image_chooser_set_image_data 
+			(GOSSIP_IMAGE_CHOOSER (dialog->avatar_chooser), NULL, 0);
 	}
 
 	gtk_widget_destroy (widget);
@@ -198,10 +197,11 @@ vcard_dialog_avatar_clicked_cb (GtkWidget         *button,
 
  	gtk_dialog_set_default_response (GTK_DIALOG (chooser_dialog), GTK_RESPONSE_ACCEPT);
 
-	path = gconf_client_get_string (gossip_app_get_gconf_client (),
-					GCONF_UI_AVATAR_DIRECTORY,
-					NULL);
-
+	path = NULL;
+	gossip_conf_get_string (gossip_conf_get (),
+				 GOSSIP_PREFS_UI_AVATAR_DIRECTORY,
+				 &path);
+	
  	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser_dialog),
 					     path ? path : g_get_home_dir ());
 	g_free (path);
@@ -224,6 +224,7 @@ vcard_dialog_avatar_clicked_cb (GtkWidget         *button,
 	gtk_widget_show (chooser_dialog);
 }
 
+#ifdef HAVE_GNOME
 static GdkPixbuf *
 vcard_dialog_scale_down_to_width (GdkPixbuf *pixbuf, gint wanted_width)
 {
@@ -246,11 +247,13 @@ vcard_dialog_scale_down_to_width (GdkPixbuf *pixbuf, gint wanted_width)
 
 	return g_object_ref (pixbuf);
 }
+#endif
 
 static void 
 vcard_dialog_avatar_update_preview_cb (GtkFileChooser    *chooser,
 				       GossipVCardDialog *dialog) 
 {
+#ifdef HAVE_GNOME
 	gchar *uri;
 
 	uri = gtk_file_chooser_get_preview_uri (chooser);
@@ -262,9 +265,10 @@ vcard_dialog_avatar_update_preview_cb (GtkFileChooser    *chooser,
 		gchar     *mime_type;
 
 		if (!dialog->thumbs) {
-			dialog->thumbs = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
+			dialog->thumbs =
+				gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
 		}
-		
+
 		mime_type = gnome_vfs_get_mime_type (uri);
 		pixbuf = gnome_thumbnail_factory_generate_thumbnail (dialog->thumbs,
 								     uri,
@@ -287,6 +291,7 @@ vcard_dialog_avatar_update_preview_cb (GtkFileChooser    *chooser,
 	}
 
 	gtk_file_chooser_set_preview_widget_active (chooser, TRUE);
+#endif
 }
 
 static void
@@ -348,17 +353,12 @@ vcard_dialog_lookup_stop (GossipVCardDialog *dialog)
 		g_source_remove (dialog->timeout_id);
 		dialog->timeout_id = 0;
 	}
-
-	if (dialog->saved_id != 0) {
-		g_source_remove (dialog->saved_id);
-		dialog->saved_id = 0;
-	}
 }
 
 static void
 vcard_dialog_get_vcard_cb (GossipResult       result,
 			   GossipVCard       *vcard,
-			   GossipVCardDialog *dialog)
+			   GossipVCardDialog *user_data)
 {
 	GtkComboBox   *combo_box;
 	GtkTextBuffer *buffer;
@@ -366,12 +366,16 @@ vcard_dialog_get_vcard_cb (GossipResult       result,
 	const guchar  *avatar;
 	gsize          avatar_size;
 
-	DEBUG_MSG (("VCardDialog: Got a VCard response"));
+	gossip_debug (DEBUG_DOMAIN, "Received VCard response");
+
+	if (!dialog) {
+		return;
+	}
 
 	vcard_dialog_lookup_stop (dialog);
 
 	if (result != GOSSIP_RESULT_OK) {
-		DEBUG_MSG (("VCardDialog: VCard result was not good"));
+		gossip_debug (DEBUG_DOMAIN, "Received VCard response was not good");
 		return;
 	}
 
@@ -400,6 +404,19 @@ vcard_dialog_get_vcard_cb (GossipResult       result,
 	dialog->last_account_selected = gtk_combo_box_get_active (combo_box);
 }
 
+static void
+vcard_dialog_set_vcard_cb (GossipResult result, 
+			   gpointer     user_data)
+{
+	gossip_debug (DEBUG_DOMAIN, "Received VCard response");
+
+	if (!dialog) {
+		return;
+	}
+	
+	gtk_widget_destroy (dialog->dialog);
+}
+
 static void
 vcard_dialog_set_vcard (GossipVCardDialog *dialog)
 {
@@ -417,7 +434,7 @@ vcard_dialog_set_vcard (GossipVCardDialog *dialog)
 	gsize                 avatar_size;
 
 	if (!gossip_app_is_connected ()) {
-		DEBUG_MSG (("VCardDialog: Not connected, not setting VCard"));
+		gossip_debug (DEBUG_DOMAIN, "Not connected, not setting VCard");
 		return;
 	}
 
@@ -453,7 +470,7 @@ vcard_dialog_set_vcard (GossipVCardDialog *dialog)
 				  account,
 				  vcard, 
 				  (GossipResultCallback) vcard_dialog_set_vcard_cb,
-				  dialog, &error);
+				  NULL, &error);
 
 	protocol = gossip_session_get_protocol (gossip_app_get_session (), account);
 	contact = gossip_protocol_get_own_contact (protocol);
@@ -464,20 +481,17 @@ vcard_dialog_set_vcard (GossipVCardDialog *dialog)
 	g_object_unref (account);
 }
 
-static void
-vcard_dialog_set_vcard_cb (GossipResult       result, 
-			   GossipVCardDialog *dialog)
-{
-	DEBUG_MSG (("VCardDialog: Got a VCard response"));
-	
-	gtk_widget_destroy (dialog->dialog);
-}
-
 static gboolean
 vcard_dialog_timeout_cb (GossipVCardDialog *dialog)
 {
 	GtkWidget *md;
 
+	gossip_debug (DEBUG_DOMAIN, "Received VCard lookup timeout");
+
+	if (!dialog) {
+		return FALSE;
+	}
+
 	vcard_dialog_lookup_stop (dialog);
 
 	/* Select last successfull account */
@@ -514,6 +528,7 @@ vcard_dialog_response_cb (GtkDialog         *widget,
 {
 	switch (response) {
 	case RESPONSE_SAVE:
+		gtk_widget_set_sensitive (dialog->button_save, FALSE);
 		vcard_dialog_set_vcard (dialog);
 		return;
 
@@ -541,20 +556,25 @@ vcard_dialog_destroy_cb (GtkWidget         *widget,
 	vcard_dialog_lookup_stop (dialog);
 
 	g_free (dialog);
+	dialog = NULL;
 }
 
 void
 gossip_vcard_dialog_show (GtkWindow *parent)
 {
-	GossipSession     *session;
-	GossipVCardDialog *dialog;
-	GladeXML          *glade;
-	GList             *accounts;
-	GtkSizeGroup      *size_group;
+	GossipSession *session;
+	GladeXML      *glade;
+	GList         *accounts;
+	GtkSizeGroup  *size_group;
+
+	if (dialog) {
+		gtk_window_present (GTK_WINDOW (dialog->dialog));
+		return;
+	}
 
 	dialog = g_new0 (GossipVCardDialog, 1);
 
-	glade = gossip_glade_get_file (GLADEDIR "/main.glade",
+	glade = gossip_glade_get_file ("main.glade",
 				       "vcard_dialog",
 				       NULL,
 				       "vcard_dialog", &dialog->dialog,
@@ -574,7 +594,6 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 				       "button_cancel", &dialog->button_cancel,
 				       "button_save", &dialog->button_save,
 				       "button_image", &dialog->button_image,
-				       "avatar_chooser", &dialog->avatar_chooser,
 				       NULL);
 
 	gossip_glade_connect (glade, 
@@ -583,8 +602,14 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 			      "vcard_dialog", "response", vcard_dialog_response_cb,
 			      NULL);
 
+	g_object_add_weak_pointer (G_OBJECT (dialog->dialog), (gpointer) &dialog);
+
 	g_object_unref (glade);
 
+	dialog->avatar_chooser = vcard_dialog_create_avatar_chooser ();
+	gtk_container_add (GTK_CONTAINER (dialog->button_image), dialog->avatar_chooser);
+	
+	/* Line up widgets */
 	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
 	gtk_size_group_add_widget (size_group, dialog->label_account);
@@ -624,9 +649,10 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 		gtk_widget_hide (dialog->hbox_account);
 	}
 
-	g_list_foreach (accounts, (GFunc)g_object_unref, NULL);
+	g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
 	g_list_free (accounts);
 
+	/* Set up transient parent */
 	if (parent) {
 		gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog), parent); 
 	}
@@ -636,3 +662,4 @@ gossip_vcard_dialog_show (GtkWindow *parent)
 	vcard_dialog_lookup_start (dialog);
 }
 
+
